Method: Chef::ActionCollection#cookbook_compilation_start

Defined in:
lib/chef/action_collection.rb

#cookbook_compilation_start(run_context) ⇒ Object

This hook gives us the run_context immediately after it is created so that we can wire up this object to it.

(see EventDispatch::Base#)



127
128
129
130
131
132
# File 'lib/chef/action_collection.rb', line 127

def cookbook_compilation_start(run_context)
  run_context.action_collection = self
  # this hook is now poorly named since it is just a callback that lets other consumers snag a reference to the action_collection
  run_context.events.enqueue(:action_collection_registration, self)
  @run_context = run_context
end