Method: Chef::CookbookVersion#identifier
- Defined in:
- lib/chef/cookbook_version.rb
#identifier ⇒ Object
This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.
The identifier field is used for cookbook_artifacts, which are organized on the chef server according to their content. If the policy_mode option to CookbookManifest is set to true it will include this field in the manifest Hash and in the upload URL.
This field may be removed or have different behavior in the future, don’t use it in 3rd party code.
65 66 67 |
# File 'lib/chef/cookbook_version.rb', line 65 def identifier @identifier end |