Method: Chef::EventDispatch::Base#msg

Defined in:
lib/chef/event_dispatch/base.rb

#msg(message) ⇒ Object

An uncategorized message. This supports the case that a user needs to pass output that doesn’t fit into one of the callbacks above. Note that there’s no semantic information about the content or importance of the message. That means that if you’re using this too often, you should add a callback for it.



388
# File 'lib/chef/event_dispatch/base.rb', line 388

def msg(message); end