Method: Chef::EventDispatch::Base#whyrun_assumption

Defined in:
lib/chef/event_dispatch/base.rb

#whyrun_assumption(action, resource, message) ⇒ Object

Called when a provider makes an assumption after a failed assertion in whyrun mode, in order to allow execution to continue



376
# File 'lib/chef/event_dispatch/base.rb', line 376

def whyrun_assumption(action, resource, message); end