41
42
43
44
45
46
47
48
49
50
51
52
53
54
55
56
57
58
59
60
61
62
63
64
65
66
67
68
69
70
71
72
73
74
75
|
# File 'lib/chef/formatters/error_inspectors/api_error_formatting.rb', line 41
def describe_eof_error(error_description)
error_description.section("Authentication Error:", " Received an EOF on transport socket. This almost always indicates a network\n error external to \#{ChefUtils::Dist::Infra::CLIENT}. Some causes include:\n\n - Blocking ICMP Dest Unreachable (breaking Path MTU Discovery)\n - IPsec or VPN tunnelling / TCP Encapsulation MTU issues\n - Jumbo frames configured only on one side (breaking Path MTU)\n - Jumbo frames configured on a LAN that does not support them\n - Proxies or Load Balancers breaking large POSTs\n - Broken TCP offload in network drivers/hardware\n\n Try sending large pings to the destination:\n\n windows: ping server.example.com -f -l 9999\n unix: ping server.example.com -s 9999\n\n Try sending large POSTs to the destination (any HTTP code returned is success):\n\n e.g.: curl http://server.example.com/`printf '%*s' 9999 '' | tr ' ' 'a'`\n\n Try disabling TCP Offload Engines (TOE) in your ethernet drivers.\n\n windows:\n Disable-NetAdapterChecksumOffload * -TcpIPv4 -UdpIPv4 -IpIPv4 -NoRestart\n Disable-NetAdapterLso * -IPv4 -NoRestart\n Set-NetAdapterAdvancedProperty * -DisplayName \"Large Receive Offload (IPv4)\" -DisplayValue Disabled \u2013NoRestart\n Restart-NetAdapter *\n unix(bash):\n for i in rx tx sg tso ufo gso gro lro rxvlan txvlan rxhash; do /sbin/ethtool -K eth0 $i off; done\n\n In some cases the underlying virtualization layer (Xen, VMware, KVM, Hyper-V, etc) may have\n broken virtual networking code.\n E\nend\n")
|