Method: Chef::Formatters::Minimal#resource_update_applied

Defined in:
lib/chef/formatters/minimal.rb

#resource_update_applied(resource, action, update) ⇒ Object

Called when a change has been made to a resource. May be called multiple times per resource, e.g., a file may have its content updated, and then its permissions updated.



188
189
190
# File 'lib/chef/formatters/minimal.rb', line 188

def resource_update_applied(resource, action, update)
  @updates_by_resource[resource.name] << Array(update)[0]
end