Method: Chef::HTTP::Decompressor#gzip_disabled?

Defined in:
lib/chef/http/decompressor.rb

#gzip_disabled?Boolean

gzip is disabled using the disable_gzip => true option in the constructor. When gzip is disabled, no ‘Accept-Encoding’ header will be set, and the response will not be decompressed, no matter what the Content-Encoding header of the response is. The intended use case for this is to work around situations where you request file.tar.gz, but the server responds with a content type of tar and a content encoding of gzip, tricking the client into decompressing the response so you end up with a tar archive (no gzip) named file.tar.gz

Returns:

  • (Boolean)


126
127
128
# File 'lib/chef/http/decompressor.rb', line 126

def gzip_disabled?
  @disable_gzip
end