Method: Chef::Node::Attribute#merged_attributes

Defined in:
lib/chef/node/attribute.rb

#merged_attributes(*path) ⇒ Object

Accessing merged attributes.

Note that merged_attributes(‘foo’, ‘bar’, ‘baz’) can be called to compute only the deep merge of node[‘bar’], but in practice we currently always compute all of node even if the user only requires node[’bar’].



414
415
416
# File 'lib/chef/node/attribute.rb', line 414

def merged_attributes(*path)
  merge_all(path)
end