Method: Chef::Provider.include_resource_dsl?

Defined in:
lib/chef/provider.rb

.include_resource_dsl?Boolean

Include attributes, public and protected methods from this Resource in the provider.

If this is set to true, delegate methods are included in the provider so that you can call (for example) attrname and it will call new_resource.attrname.

The actual include does not happen until the first time the Provider is instantiated (so that we don’t have to worry about load order issues).

Parameters:

  • include_resource_dsl (Boolean)

    Whether to include resource DSL or not (defaults to false).

Returns:

  • (Boolean)


417
418
419
# File 'lib/chef/provider.rb', line 417

def self.include_resource_dsl?
  false
end