Method: Chef::Provider#load_after_resource

Defined in:
lib/chef/provider.rb

#load_after_resourceObject



193
194
195
196
197
198
# File 'lib/chef/provider.rb', line 193

def load_after_resource
  # This is a backwards compatible hack, custom resources properly wire up a new after_resource
  # via load_current_value.  It is acceptable for old style resources that cannot be easily made
  # into custom resources to override this method and provide a proper after_resource.
  @after_resource = @new_resource
end