Method: Chef::Provider#validate_required_properties!
- Defined in:
- lib/chef/provider.rb
#validate_required_properties! ⇒ Object
210 211 212 213 |
# File 'lib/chef/provider.rb', line 210 def validate_required_properties! # all we do is run through all the required properties for this action and vivify them new_resource.class.properties.each { |name, property| property.required?(action) && property.get(new_resource) } end |