Method: Chef::Resource::Execute#is_guard_interpreter

Defined in:
lib/chef/resource/execute.rb

#is_guard_interpreterObject

The ResourceGuardInterpreter wraps a resource’s guards in another resource. That inner resource needs to behave differently during (for example) why_run mode, so we flag it here. For why_run mode we still want to execute the guard resource even if we are not executing the wrapping resource. Only execute resources (and subclasses) can be guard interpreters.



509
510
511
# File 'lib/chef/resource/execute.rb', line 509

def is_guard_interpreter
  @is_guard_interpreter
end