Method: Chef::Resource::PowershellScript.get_default_attributes

Defined in:
lib/chef/resource/powershell_script.rb

.get_default_attributesObject

Allow callers evaluating guards to request default attribute values. This is needed to allow convert_boolean_return to be true in guard context by default, and false by default otherwise. When this mode becomes the default for this resource, this method can be removed since guard context and recipe resource context will have the same behavior.



82
83
84
# File 'lib/chef/resource/powershell_script.rb', line 82

def self.get_default_attributes
  { convert_boolean_return: true }
end