Method: Chef::Resource#declared_type

Defined in:
lib/chef/resource.rb

#declared_typeString

The actual name that was used to create this resource. Sometimes, when you say something like ‘package ’blah’‘, the system will create a different resource (i.e. YumPackage). When this happens, the user will expect to see the thing they wrote, not the type that was returned. May be nil, in which case callers should read #resource_name. See #declared_key.

Returns:

  • (String)

    The actual name that was used to create this resource. Sometimes, when you say something like ‘package ’blah’‘, the system will create a different resource (i.e. YumPackage). When this happens, the user will expect to see the thing they wrote, not the type that was returned. May be nil, in which case callers should read #resource_name. See #declared_key.



1274
1275
1276
# File 'lib/chef/resource.rb', line 1274

def declared_type
  @declared_type
end