Method: Chef::Resource#enclosing_provider

Defined in:
lib/chef/resource.rb

#enclosing_providerChef::Provider

Returns The provider this resource was declared in (if it was declared in an LWRP). When you call methods that do not exist on this Resource, Chef will try to call the method on the provider as well before giving up.

Returns:

  • (Chef::Provider)

    The provider this resource was declared in (if it was declared in an LWRP). When you call methods that do not exist on this Resource, Chef will try to call the method on the provider as well before giving up.



1257
1258
1259
# File 'lib/chef/resource.rb', line 1257

def enclosing_provider
  @enclosing_provider
end