Method: Chef::Resource#ignore_failure

Defined in:
lib/chef/resource.rb

#ignore_failureObject

Whether to ignore failures. If set to true, and this resource when an action is run, the resource will be marked as failed but no exception will be thrown (and no error will be output). Defaults to false. If set to :quiet or ‘’quiet’‘, the normal error trace will be suppressed.

TODO ignore_failure and retries seem to be mutually exclusive; I doubt that was intended.

Parameters:

  • arg (Boolean, String, Symbol)

    Whether to ignore failures.

Returns:

  • Whether this resource will ignore failures.



549
# File 'lib/chef/resource.rb', line 549

property :ignore_failure, [ true, false, :quiet, "quiet" ], default: false, desired_state: false