Method: Chef::Resource#initialize

Defined in:
lib/chef/resource.rb

#initialize(name, run_context = nil) ⇒ Resource

Create a new Resource.

Parameters:

  • name

    The name of this resource (corresponds to the #name attribute, used for notifications to this resource).

  • run_context (defaults to: nil)

    The context of the Chef run. Corresponds to #run_context.



111
112
113
114
115
116
117
118
119
120
121
122
123
124
125
126
127
128
129
130
131
132
133
134
135
136
137
138
139
140
141
142
# File 'lib/chef/resource.rb', line 111

def initialize(name, run_context = nil)
  name(name) unless name.nil?
  @run_context = run_context

  @logger = if run_context
              run_context.logger.with_child({ name: name, resource: resource_name })
            else
              Chef::Log.with_child({ name: name, resource: resource_name })
            end

  @before = nil
  @params = {}
  @provider = nil
  @allowed_actions = self.class.allowed_actions.to_a
  @action = self.class.default_action
  @updated = false
  @updated_by_last_action = false
  @not_if = []
  @only_if = []
  @source_line = nil
  @deprecated = false
  @skip_docs = false
  # We would like to raise an error when the user gives us a guard
  # interpreter and a ruby_block to the guard. In order to achieve this
  # we need to understand when the user overrides the default guard
  # interpreter. Therefore we store the default separately in a different
  # attribute.
  @guard_interpreter = nil
  @default_guard_interpreter = :default
  @elapsed_time = 0
  @executed_by_runner = false
end