Method: Chef::Resource#lookup_provider_constant

Defined in:
lib/chef/resource.rb

#lookup_provider_constant(name, action = :nothing) ⇒ Object

This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.



1634
1635
1636
1637
1638
1639
1640
1641
1642
1643
# File 'lib/chef/resource.rb', line 1634

def lookup_provider_constant(name, action = :nothing)
  # XXX: "name" is probably a poor choice of name here, ideally this would be nil, but we need to
  # fix resources so that nil or empty names work (also solving the apt_update "doesn't matter one bit"
  # problem).  WARNING: this string is not a public API and should not be referenced (e.g. in provides blocks)
  # and may change at any time.  If you've found this comment you're also probably very lost and should maybe
  # consider using `declare_resource :whatever` instead of trying to set `provider :whatever` on a resource, or in some
  # other way reconsider what you're trying to do, since you're likely trying to force a bad design that we
  # can't/won't support.
  self.class.resource_for_node(name, node).new("name", run_context).provider_for_action(action).class
end