Method: Chef::Resource#retries

Defined in:
lib/chef/resource.rb

#retriesInteger

The number of times to retry this resource if it fails by throwing an exception while running an action. Default: 0

When the retries have run out, the Resource will throw the last exception.

Parameters:

  • arg (Integer)

    The number of retries.

Returns:

  • (Integer)

    The number of retries.



425
# File 'lib/chef/resource.rb', line 425

property :retries, Integer, default: 0, desired_state: false