Method: Chef::Resource#should_skip?

Defined in:
lib/chef/resource.rb

#should_skip?(action) ⇒ Boolean

Evaluates not_if and only_if conditionals. Returns a falsey value if any of the conditionals indicate that this resource should be skipped, i.e., if an only_if evaluates to false or a not_if evaluates to true.

If this resource should be skipped, returns the first conditional that “fails” its check. Subsequent conditionals are not evaluated, so in general it’s not a good idea to rely on side effects from not_if or only_if commands/blocks being evaluated.

Also skips conditional checking when the action is :nothing

Returns:



1584
1585
1586
1587
1588
1589
1590
1591
1592
1593
1594
1595
1596
1597
# File 'lib/chef/resource.rb', line 1584

def should_skip?(action)
  conditional_action = ConditionalActionNotNothing.new(action)

  conditionals = [ conditional_action ] + only_if + not_if
  conditionals.find do |conditional|
    if conditional.continue?
      false
    else
      events.resource_skipped(self, action, conditional)
      logger.debug("Skipping #{self} due to #{conditional.description}")
      true
    end
  end
end