Method: Chef::Resource#subscribes

Defined in:
lib/chef/resource.rb

#subscribes(action, resources, timing = :delayed) ⇒ Object

Subscribes to updates from other resources, causing a particular action to run on this resource when the other resource is updated.

If multiple resources are specified, this resource action will be run if any of them change.

This notification will only trigger once, no matter how many other resources are updated (or how many actions are run by a particular resource).

Examples:

Resources by string

file '/foo.txt' do
  content 'hi'
  action :nothing
  subscribes :create, 'file[/bar.txt]'
end
file '/bar.txt' do
  content 'hi'
end

Direct resource

bar = file '/bar.txt' do
  content 'hi'
end
file '/foo.txt' do
  content 'hi'
  action :nothing
  subscribes :create, bar
end

Multiple resources by string

file '/foo.txt' do
  content 'hi'
  action :nothing
  subscribes :create, [ 'file[/bar.txt]', 'file[/baz.txt]' ]
end
file '/bar.txt' do
  content 'hi'
end
file '/baz.txt' do
  content 'hi'
end

Multiple resources

bar = file '/bar.txt' do
  content 'hi'
end
baz = file '/bar.txt' do
  content 'hi'
end
file '/foo.txt' do
  content 'hi'
  action :nothing
  subscribes :create, [ bar, baz ]
end

Parameters:

  • action

    The action to run on the other resource.

  • resources (String, Resource, Array[String, Resource])

    The resources to subscribe to.

  • timing (String, Symbol) (defaults to: :delayed)

    When to notify. Has these values:

    • delayed: An update will cause the action to run after all other actions have been run. This is the default.

    • immediate, immediately: The action will run immediately following the other resource being updated.

    • before: The action will run immediately before the other resource is updated.



341
342
343
344
345
346
347
348
349
350
351
352
353
354
# File 'lib/chef/resource.rb', line 341

def subscribes(action, resources, timing = :delayed)
  resources = [resources].flatten
  resources.each do |resource|
    validate_resource_spec!(resource)
    if resource.is_a?(String)
      resource = UnresolvedSubscribes.new(resource, run_context)
    end
    if resource.run_context.nil?
      resource.run_context = run_context
    end
    resource.notifies(action, self, timing)
  end
  true
end