Method: Chef::Resource#updated?

Defined in:
lib/chef/resource.rb

#updated?Boolean

Whether or not this resource was updated during an action. If multiple actions are set on the resource, this will be true if any action caused an update to happen.

Returns:

  • (Boolean)

    Whether the resource was updated during an action.



915
916
917
# File 'lib/chef/resource.rb', line 915

def updated?
  updated
end