Method: Chef::ResourceCollection::ResourceList#insert

Defined in:
lib/chef/resource_collection/resource_list.rb

#insert(resource) ⇒ Object

If @insert_after_idx is nil, we are not currently executing a converge so the Resource is appended to the end of the list. If @insert_after_idx is NOT nil, we ARE currently executing a converge so the resource is inserted into the middle of the list after the last resource that was converged. If it is called multiple times (when an LWRP contains multiple resources) it keeps track of that. See this example ResourceList:

File1, LWRP1, File2

# The iterator starts and points to File1. It is executed and @insert_after_idx=0

File1, LWRP1, File2

# The iterator moves to LWRP1. It is executed and @insert_after_idx=1

File1, LWRP1, Service1, File2

# The LWRP execution inserts Service1 and @insert_after_idx=2

File1, LWRP1, Service1, Service2, File2

# The LWRP inserts Service2 and @insert_after_idx=3. The LWRP

finishes executing
File1, LWRP1, Service1, Service2, File2

# The iterator moves to Service1 since it is the next non-executed

resource.  The execute_each_resource call below resets @insert_after_idx=2

If Service1 was another LWRP, it would insert its resources between Service1 and Service2. The iterator keeps track of executed resources and @insert_after_idx keeps track of where the next resource to insert should be.

Parameters:



61
62
63
64
65
66
67
68
# File 'lib/chef/resource_collection/resource_list.rb', line 61

def insert(resource)
  is_chef_resource!(resource)
  if @insert_after_idx
    @resources.insert(@insert_after_idx += 1, resource)
  else
    @resources << resource
  end
end