Method: Chef::RunContext#before_notification_collection

Defined in:
lib/chef/run_context.rb

#before_notification_collectionHash[String, Array[Chef::Resource::Notification]] (readonly)

A Hash containing the before notifications triggered by resources during the converge phase of the chef run.

Returns:



172
173
174
# File 'lib/chef/run_context.rb', line 172

def before_notification_collection
  @before_notification_collection
end