Method: Chef::RunContext#immediate_notification_collection

Defined in:
lib/chef/run_context.rb

#immediate_notification_collectionHash[String, Array[Chef::Resource::Notification]] (readonly)

A Hash containing the immediate notifications triggered by resources during the converge phase of the chef run.

Returns:



180
181
182
# File 'lib/chef/run_context.rb', line 180

def immediate_notification_collection
  @immediate_notification_collection
end