Method: Chef::RunContext#initialize

Defined in:
lib/chef/run_context.rb

#initialize(node = nil, cookbook_collection = nil, events = nil, logger = nil) ⇒ RunContext

Creates a new Chef::RunContext object and populates its fields. This object gets used by the Chef Server to generate a fully compiled recipe list for a node.

Parameters:



222
223
224
225
226
227
228
229
230
231
232
233
234
235
236
237
238
239
# File 'lib/chef/run_context.rb', line 222

def initialize(node = nil, cookbook_collection = nil, events = nil, logger = nil)
  @events = events
  @logger = logger || Chef::Log.with_child
  self.node = node if node
  self.cookbook_collection = cookbook_collection if cookbook_collection
  @definitions = {}
  @loaded_recipes_hash = {}
  @loaded_attributes_hash = {}
  @reboot_info = {}
  @cookbook_compiler = nil
  @input_collection = Chef::Compliance::InputCollection.new(events)
  @waiver_collection = Chef::Compliance::WaiverCollection.new(events)
  @profile_collection = Chef::Compliance::ProfileCollection.new(events)
  @default_secret_service = nil
  @default_secret_config = {}

  initialize_child_state
end