Method: Chef::RunContext#loaded_attributes

Defined in:
lib/chef/run_context.rb

#loaded_attributesArray[String]

A list of all attributes files that have been loaded.

Stored internally using a Hash, so order is predictable.

TODO is the above statement true in a 1.9+ ruby world? Is it relevant?

Returns:

  • (Array[String])

    A list of attribute file names in fully qualified form, e.g. the “nginx” will be given as “nginx::default”.



502
503
504
# File 'lib/chef/run_context.rb', line 502

def loaded_attributes
  loaded_attributes_hash.keys
end