Method: Chef::Util::FileEdit#search_file_delete

Defined in:
lib/chef/util/file_edit.rb

#search_file_delete(regex) ⇒ Object

search the file line by line and match each line with the given regex if matched, delete the match (all occurrences) from the line



64
65
66
# File 'lib/chef/util/file_edit.rb', line 64

def search_file_delete(regex)
  search_file_replace(regex, "")
end