Class: Chef::CookbookVersion

Inherits:
Object
  • Object
show all
Includes:
Comparable
Defined in:
lib/chef/cookbook_version.rb

Overview

Chef::CookbookVersion

CookbookVersion is a model object encapsulating the data about a Chef cookbook. Chef supports maintaining multiple versions of a cookbook on a single server; each version is represented by a distinct instance of this class. – TODO: timh/cw: 5-24-2010: mutators for files (e.g., recipe_filenames=, recipe_filenames.insert) should dirty the manifest so it gets regenerated.

Constant Summary collapse

COOKBOOK_SEGMENTS =
[ :resources, :providers, :recipes, :definitions, :libraries, :attributes, :files, :templates, :root_files ]

Instance Attribute Summary collapse

Class Method Summary collapse

Instance Method Summary collapse

Constructor Details

#initialize(name) ⇒ CookbookVersion

Creates a new Chef::CookbookVersion object.

Returns

object<Chef::CookbookVersion>

Duh. :)



86
87
88
89
90
91
92
93
94
95
96
97
98
99
100
101
102
103
104
105
# File 'lib/chef/cookbook_version.rb', line 86

def initialize(name)
  @name = name
  @frozen = false
  @attribute_filenames = Array.new
  @definition_filenames = Array.new
  @template_filenames = Array.new
  @file_filenames = Array.new
  @recipe_filenames = Array.new
  @recipe_filenames_by_name = Hash.new
  @library_filenames = Array.new
  @resource_filenames = Array.new
  @provider_filenames = Array.new
   = Array.new
  @root_dir = nil
  @root_filenames = Array.new
  @status = :ready
  @manifest = nil
  @file_vendor = nil
   = Chef::Cookbook::.new
end

Instance Attribute Details

#attribute_filenamesObject Also known as: attribute_files

attribute_filenames also has a setter that has non-default functionality.



60
61
62
# File 'lib/chef/cookbook_version.rb', line 60

def attribute_filenames
  @attribute_filenames
end

#attribute_filenames_by_short_filenameObject (readonly)

Returns the value of attribute attribute_filenames_by_short_filename.



67
68
69
# File 'lib/chef/cookbook_version.rb', line 67

def attribute_filenames_by_short_filename
  @attribute_filenames_by_short_filename
end

#definition_filenamesObject

Returns the value of attribute definition_filenames.



46
47
48
# File 'lib/chef/cookbook_version.rb', line 46

def definition_filenames
  @definition_filenames
end

#file_filenamesObject

Returns the value of attribute file_filenames.



48
49
50
# File 'lib/chef/cookbook_version.rb', line 48

def file_filenames
  @file_filenames
end

#library_filenamesObject

Returns the value of attribute library_filenames.



49
50
51
# File 'lib/chef/cookbook_version.rb', line 49

def library_filenames
  @library_filenames
end

#metadataObject

Returns the value of attribute metadata.



54
55
56
# File 'lib/chef/cookbook_version.rb', line 54

def 
  
end

#metadata_filenamesObject

Returns the value of attribute metadata_filenames.



55
56
57
# File 'lib/chef/cookbook_version.rb', line 55

def 
  
end

#nameObject

Returns the value of attribute name.



53
54
55
# File 'lib/chef/cookbook_version.rb', line 53

def name
  @name
end

#provider_filenamesObject

Returns the value of attribute provider_filenames.



51
52
53
# File 'lib/chef/cookbook_version.rb', line 51

def provider_filenames
  @provider_filenames
end

#recipe_filenamesObject Also known as: recipe_files

recipe_filenames also has a setter that has non-default functionality.



64
65
66
# File 'lib/chef/cookbook_version.rb', line 64

def recipe_filenames
  @recipe_filenames
end

#recipe_filenames_by_nameObject (readonly)

Returns the value of attribute recipe_filenames_by_name.



66
67
68
# File 'lib/chef/cookbook_version.rb', line 66

def recipe_filenames_by_name
  @recipe_filenames_by_name
end

#resource_filenamesObject

Returns the value of attribute resource_filenames.



50
51
52
# File 'lib/chef/cookbook_version.rb', line 50

def resource_filenames
  @resource_filenames
end

#root_dirObject

Returns the value of attribute root_dir.



45
46
47
# File 'lib/chef/cookbook_version.rb', line 45

def root_dir
  @root_dir
end

#root_filenamesObject

Returns the value of attribute root_filenames.



52
53
54
# File 'lib/chef/cookbook_version.rb', line 52

def root_filenames
  @root_filenames
end

#statusObject

Returns the value of attribute status.



56
57
58
# File 'lib/chef/cookbook_version.rb', line 56

def status
  @status
end

#template_filenamesObject

Returns the value of attribute template_filenames.



47
48
49
# File 'lib/chef/cookbook_version.rb', line 47

def template_filenames
  @template_filenames
end

Class Method Details

.available_versions(cookbook_name) ⇒ Object

Given a cookbook_name, get a list of all versions that exist on the server.

Returns

[String]

Array of cookbook versions, which are strings like ‘x.y.z’

nil

if the cookbook doesn’t exist. an error will also be logged.



542
543
544
545
546
547
548
549
550
551
552
553
# File 'lib/chef/cookbook_version.rb', line 542

def self.available_versions(cookbook_name)
  chef_server_rest.get_rest("cookbooks/#{cookbook_name}")[cookbook_name]["versions"].map do |cb|
    cb["version"]
  end
rescue Net::HTTPServerException => e
  if e.to_s =~ /^404/
    Chef::Log.error("Cannot find a cookbook named #{cookbook_name}")
    nil
  else
    raise
  end
end

.cacheObject



78
79
80
# File 'lib/chef/cookbook_version.rb', line 78

def self.cache
  Chef::FileCache
end

.checksum_cookbook_file(filepath) ⇒ Object

This is the one and only method that knows how cookbook files’ checksums are generated.



71
72
73
74
75
76
# File 'lib/chef/cookbook_version.rb', line 71

def self.checksum_cookbook_file(filepath)
  Chef::Digester.generate_md5_checksum_for_file(filepath)
rescue Errno::ENOENT
  Chef::Log.debug("File #{filepath} does not exist, so there is no checksum to generate")
  nil
end

.chef_server_restObject

REST API



495
496
497
# File 'lib/chef/cookbook_version.rb', line 495

def self.chef_server_rest
  Chef::REST.new(Chef::Config[:chef_server_url])
end

.json_create(o) ⇒ Object



452
453
454
455
456
457
458
459
460
461
462
463
# File 'lib/chef/cookbook_version.rb', line 452

def self.json_create(o)
  cookbook_version = new(o["cookbook_name"])
  # We want the Chef::Cookbook::Metadata object to always be inflated
  cookbook_version. = Chef::Cookbook::.from_hash(o["metadata"])
  cookbook_version.manifest = o

  # We don't need the following step when we decide to stop supporting deprecated operators in the metadata (e.g. <<, >>)
  cookbook_version.manifest["metadata"] = Chef::JSONCompat.from_json(cookbook_version..to_json)

  cookbook_version.freeze_version if o["frozen?"]
  cookbook_version
end

.latest_cookbooksObject

Get the newest version of all cookbooks



556
557
558
# File 'lib/chef/cookbook_version.rb', line 556

def self.latest_cookbooks
  chef_server_rest.get_rest('cookbooks/_latest')
end

.listObject

The API returns only a single version of each cookbook in the result from the cookbooks method



528
529
530
# File 'lib/chef/cookbook_version.rb', line 528

def self.list
  chef_server_rest.get_rest('cookbooks')
end

.list_all_versionsObject



532
533
534
# File 'lib/chef/cookbook_version.rb', line 532

def self.list_all_versions
  chef_server_rest.get_rest('cookbooks?num_versions=all')
end

.load(name, version = "_latest") ⇒ Object



522
523
524
525
# File 'lib/chef/cookbook_version.rb', line 522

def self.load(name, version="_latest")
  version = "_latest" if version == "latest"
  chef_server_rest.get_rest("cookbooks/#{name}/#{version}")
end

Instance Method Details

#<=>(o) ⇒ Object



560
561
562
563
564
565
566
# File 'lib/chef/cookbook_version.rb', line 560

def <=>(o)
  raise Chef::Exceptions::CookbookVersionNameMismatch if self.name != o.name
  # FIXME: can we change the interface to the Metadata class such
  # that metadata.version returns a Chef::Version instance instead
  # of a string?
  Chef::Version.new(self.version) <=> Chef::Version.new(o.version)
end

#checksumsObject

Returns a hash of checksums to either nil or the on disk path (which is done by generate_manifest).



178
179
180
181
182
183
# File 'lib/chef/cookbook_version.rb', line 178

def checksums
  unless @checksums
    generate_manifest
  end
  @checksums
end

#chef_server_restObject



499
500
501
# File 'lib/chef/cookbook_version.rb', line 499

def chef_server_rest
  self.class.chef_server_rest
end

#destroyObject



517
518
519
520
# File 'lib/chef/cookbook_version.rb', line 517

def destroy
  chef_server_rest.delete_rest("cookbooks/#{name}/#{version}")
  self
end

#force_save_urlObject

Adds the ‘force=true` parameter to the upload URL. This allows the user to overwrite a frozen cookbook (a PUT against the normal #save_url raises a 409 Conflict in this case).



513
514
515
# File 'lib/chef/cookbook_version.rb', line 513

def force_save_url
  "cookbooks/#{name}/#{version}?force=true"
end

#freeze_versionObject



118
119
120
# File 'lib/chef/cookbook_version.rb', line 118

def freeze_version
  @frozen = true
end

#frozen_version?Boolean

Indicates if this version is frozen or not. Freezing a coobkook version indicates that a new cookbook with the same name and version number shoule

Returns:

  • (Boolean)


114
115
116
# File 'lib/chef/cookbook_version.rb', line 114

def frozen_version?
  @frozen
end

#full_nameObject



190
191
192
# File 'lib/chef/cookbook_version.rb', line 190

def full_name
  "#{name}-#{version}"
end

#fully_qualified_recipe_namesObject

Return recipe names in the form of cookbook_name::recipe_name



205
206
207
208
209
210
211
# File 'lib/chef/cookbook_version.rb', line 205

def fully_qualified_recipe_names
  results = Array.new
  recipe_filenames_by_name.each_key do |rname|
    results << "#{name}::#{rname}"
  end
  results
end

#generate_manifest_with_urls(&url_generator) ⇒ Object



465
466
467
468
469
470
471
472
473
474
475
476
# File 'lib/chef/cookbook_version.rb', line 465

def generate_manifest_with_urls(&url_generator)
  rendered_manifest = manifest.dup
  COOKBOOK_SEGMENTS.each do |segment|
    if rendered_manifest.has_key?(segment)
      rendered_manifest[segment].each do |manifest_record|
        url_options = { :cookbook_name => name.to_s, :cookbook_version => version, :checksum => manifest_record["checksum"] }
        manifest_record["url"] = url_generator.call(url_options)
      end
    end
  end
  rendered_manifest
end

#load_recipe(recipe_name, run_context) ⇒ Object

called from DSL



224
225
226
227
228
229
230
231
232
233
234
235
236
237
238
239
# File 'lib/chef/cookbook_version.rb', line 224

def load_recipe(recipe_name, run_context)
  unless recipe_filenames_by_name.has_key?(recipe_name)
    raise Chef::Exceptions::RecipeNotFound, "could not find recipe #{recipe_name} for cookbook #{name}"
  end

  Chef::Log.debug("Found recipe #{recipe_name} in cookbook #{name}")
  recipe = Chef::Recipe.new(name, recipe_name, run_context)
  recipe_filename = recipe_filenames_by_name[recipe_name]

  unless recipe_filename
    raise Chef::Exceptions::RecipeNotFound, "could not find #{recipe_name} files for cookbook #{name}"
  end

  recipe.from_file(recipe_filename)
  recipe
end

#manifestObject

A manifest is a Mash that maps segment names to arrays of manifest records (see #preferred_manifest_record for format of manifest records), as well as describing cookbook metadata. The manifest follows a form like the following:

{
  :cookbook_name = "apache2",
  :version = "1.0",
  :name = "Apache 2"
  :metadata = ???TODO: timh/cw: 5-24-2010: describe this format,

  :files => [
    {
      :name => "afile.rb",
      :path => "files/ubuntu-9.10/afile.rb",
      :checksum => "2222",
      :specificity => "ubuntu-9.10"
    },
  ],
  :templates => [ manifest_record1, ... ],
  ...
}


149
150
151
152
153
154
# File 'lib/chef/cookbook_version.rb', line 149

def manifest
  unless @manifest
    generate_manifest
  end
  @manifest
end

#manifest=(new_manifest) ⇒ Object



156
157
158
159
160
161
162
163
164
165
166
167
168
169
170
171
172
173
174
# File 'lib/chef/cookbook_version.rb', line 156

def manifest=(new_manifest)
  @manifest = Mash.new new_manifest
  @checksums = extract_checksums_from_manifest(@manifest)
  @manifest_records_by_path = extract_manifest_records_by_path(@manifest)

  COOKBOOK_SEGMENTS.each do |segment|
    next unless @manifest.has_key?(segment)
    filenames = @manifest[segment].map{|manifest_record| manifest_record['name']}

    if segment == :recipes
      self.recipe_filenames = filenames
    elsif segment == :attributes
      self.attribute_filenames = filenames
    else
      segment_filenames(segment).clear
      filenames.each { |filename| segment_filenames(segment) << filename }
    end
  end
end

#manifest_records_by_pathObject



185
186
187
188
# File 'lib/chef/cookbook_version.rb', line 185

def manifest_records_by_path
  @manifest_records_by_path || generate_manifest
  @manifest_records_by_path
end

#metadata_json_fileObject



478
479
480
# File 'lib/chef/cookbook_version.rb', line 478

def 
  File.join(root_dir, "metadata.json")
end

#metadata_rb_fileObject



482
483
484
# File 'lib/chef/cookbook_version.rb', line 482

def 
  File.join(root_dir, "metadata.rb")
end

#preferred_filename_on_disk_location(node, segment, filename, current_filepath = nil) ⇒ Object



313
314
315
316
317
318
319
320
# File 'lib/chef/cookbook_version.rb', line 313

def preferred_filename_on_disk_location(node, segment, filename, current_filepath=nil)
  manifest_record = preferred_manifest_record(node, segment, filename)
  if current_filepath && (manifest_record['checksum'] == self.class.checksum_cookbook_file(current_filepath))
    nil
  else
    file_vendor.get_filename(manifest_record['path'])
  end
end

#preferred_manifest_record(node, segment, filename) ⇒ Object

Determine the most specific manifest record for the given segment/filename, given information in the node. Throws FileNotFound if there is no such segment and filename in the manifest.

A manifest record is a Mash that follows the following form:

:name => "example.rb",
:path => "files/default/example.rb",
:specificity => "default",
:checksum => "1234"



280
281
282
283
284
285
286
287
288
289
290
291
292
293
294
295
296
297
298
299
300
301
302
303
304
305
306
307
308
309
310
311
# File 'lib/chef/cookbook_version.rb', line 280

def preferred_manifest_record(node, segment, filename)
  preferences = preferences_for_path(node, segment, filename)

  # ensure that we generate the manifest, which will also generate
  # @manifest_records_by_path
  manifest

  # in order of prefernce, look for the filename in the manifest
  found_pref = preferences.find {|preferred_filename| @manifest_records_by_path[preferred_filename] }
  if found_pref
    @manifest_records_by_path[found_pref]
  else
    if segment == :files || segment == :templates
      error_message = "Cookbook '#{name}' (#{version}) does not contain a file at any of these locations:\n"
      error_locations = [
        "  #{segment}/#{node[:platform]}-#{node[:platform_version]}/#{filename}",
        "  #{segment}/#{node[:platform]}/#{filename}",
        "  #{segment}/default/#{filename}",
      ]
      error_message << error_locations.join("\n")
      existing_files = segment_filenames(segment)
      # Show the files that the cookbook does have. If the user made a typo,
      # hopefully they'll see it here.
      unless existing_files.empty?
        error_message << "\n\nThis cookbook _does_ contain: ['#{existing_files.join("','")}']"
      end
      raise Chef::Exceptions::FileNotFound, error_message
    else
      raise Chef::Exceptions::FileNotFound, "cookbook #{name} does not contain file #{segment}/#{filename}"
    end
  end
end

#preferred_manifest_records_for_directory(node, segment, dirname) ⇒ Object

Determine the manifest records from the most specific directory for the given node. See #preferred_manifest_record for a description of entries of the returned Array.



363
364
365
366
367
368
369
370
371
372
373
374
375
376
377
378
379
380
381
382
383
384
385
386
387
388
389
390
391
392
# File 'lib/chef/cookbook_version.rb', line 363

def preferred_manifest_records_for_directory(node, segment, dirname)
  preferences = preferences_for_path(node, segment, dirname)
  records_by_pref = Hash.new
  preferences.each { |pref| records_by_pref[pref] = Array.new }

  manifest[segment].each do |manifest_record|
    manifest_record_path = manifest_record[:path]

    # extract the preference part from the path.
    if manifest_record_path =~ /(#{Regexp.escape(segment.to_s)}\/[^\/]+\/#{Regexp.escape(dirname)})\/.+$/
      # Note the specificy_dirname includes the segment and
      # dirname argument as above, which is what
      # preferences_for_path returns. It could be
      # "files/ubuntu-9.10/dirname", for example.
      specificity_dirname = $1

      # Record the specificity_dirname only if it's in the list of
      # valid preferences
      if records_by_pref[specificity_dirname]
        records_by_pref[specificity_dirname] << manifest_record
      end
    end
  end

  best_pref = preferences.find { |pref| !records_by_pref[pref].empty? }

  raise Chef::Exceptions::FileNotFound, "cookbook #{name} (#{version}) has no directory #{segment}/default/#{dirname}" unless best_pref

  records_by_pref[best_pref]
end

#relative_filenames_in_preferred_directory(node, segment, dirname) ⇒ Object



322
323
324
325
326
327
328
329
330
331
332
333
334
335
336
337
338
339
340
341
342
343
344
345
346
347
348
349
350
351
352
353
354
355
356
357
358
# File 'lib/chef/cookbook_version.rb', line 322

def relative_filenames_in_preferred_directory(node, segment, dirname)
  preferences = preferences_for_path(node, segment, dirname)
  filenames_by_pref = Hash.new
  preferences.each { |pref| filenames_by_pref[pref] = Array.new }

  manifest[segment].each do |manifest_record|
    manifest_record_path = manifest_record[:path]

    # find the NON SPECIFIC filenames, but prefer them by filespecificity.
    # For example, if we have a file:
    # 'files/default/somedir/somefile.conf' we only keep
    # 'somedir/somefile.conf'. If there is also
    # 'files/$hostspecific/somedir/otherfiles' that matches the requested
    # hostname specificity, that directory will win, as it is more specific.
    #
    # This is clearly ugly b/c the use case is for remote directory, where
    # we're just going to make cookbook_files out of these and make the
    # cookbook find them by filespecificity again. but it's the shortest
    # path to "success" for now.
    if manifest_record_path =~ /(#{Regexp.escape(segment.to_s)}\/[^\/]+\/#{Regexp.escape(dirname)})\/.+$/
      specificity_dirname = $1
      non_specific_path = manifest_record_path[/#{Regexp.escape(segment.to_s)}\/[^\/]+\/#{Regexp.escape(dirname)}\/(.+)$/, 1]
      # Record the specificity_dirname only if it's in the list of
      # valid preferences
      if filenames_by_pref[specificity_dirname]
        filenames_by_pref[specificity_dirname] << non_specific_path
      end
    end
  end

  best_pref = preferences.find { |pref| !filenames_by_pref[pref].empty? }

  raise Chef::Exceptions::FileNotFound, "cookbook #{name} has no directory #{segment}/default/#{dirname}" unless best_pref

  filenames_by_pref[best_pref]

end

#reload_metadata!Object



486
487
488
489
490
# File 'lib/chef/cookbook_version.rb', line 486

def reload_metadata!
  if File.exists?()
    .from_json(IO.read())
  end
end

#save_urlObject

Return the URL to save (PUT) this object to the server via the REST api. If there is an existing document on the server and it is marked frozen, a PUT will result in a 409 Conflict.



506
507
508
# File 'lib/chef/cookbook_version.rb', line 506

def save_url
  "cookbooks/#{name}/#{version}"
end

#segment_filenames(segment) ⇒ Object



241
242
243
244
245
246
247
248
249
250
251
252
253
254
255
256
257
258
259
260
261
262
263
264
265
266
# File 'lib/chef/cookbook_version.rb', line 241

def segment_filenames(segment)
  unless COOKBOOK_SEGMENTS.include?(segment)
    raise ArgumentError, "invalid segment #{segment}: must be one of #{COOKBOOK_SEGMENTS.join(', ')}"
  end

  case segment.to_sym
  when :resources
    @resource_filenames
  when :providers
    @provider_filenames
  when :recipes
    @recipe_filenames
  when :libraries
    @library_filenames
  when :definitions
    @definition_filenames
  when :attributes
    @attribute_filenames
  when :files
    @file_filenames
  when :templates
    @template_filenames
  when :root_files
    @root_filenames
  end
end

#to_hashObject



439
440
441
442
443
444
# File 'lib/chef/cookbook_version.rb', line 439

def to_hash
  result = manifest.dup
  result['frozen?'] = frozen_version?
  result['chef_type'] = 'cookbook_version'
  result.to_hash
end

#to_json(*a) ⇒ Object



446
447
448
449
450
# File 'lib/chef/cookbook_version.rb', line 446

def to_json(*a)
  result = self.to_hash
  result['json_class'] = self.class.name
  result.to_json(*a)
end

#versionObject



107
108
109
# File 'lib/chef/cookbook_version.rb', line 107

def version
  .version
end

#version=(new_version) ⇒ Object



122
123
124
125
# File 'lib/chef/cookbook_version.rb', line 122

def version=(new_version)
  manifest["version"] = new_version
  .version(new_version)
end