Class: Chef::Knife
- Inherits:
-
Object
show all
- Extended by:
- Mixin::ConvertToClassName, Forwardable
- Includes:
- Mixin::PathSanity, Mixlib::CLI
- Defined in:
- lib/chef/knife.rb,
lib/chef/knife/raw.rb,
lib/chef/knife/ssh.rb,
lib/chef/knife/deps.rb,
lib/chef/knife/diff.rb,
lib/chef/knife/edit.rb,
lib/chef/knife/help.rb,
lib/chef/knife/list.rb,
lib/chef/knife/show.rb,
lib/chef/knife/xargs.rb,
lib/chef/knife/delete.rb,
lib/chef/knife/search.rb,
lib/chef/knife/status.rb,
lib/chef/knife/upload.rb,
lib/chef/knife/core/ui.rb,
lib/chef/knife/download.rb,
lib/chef/knife/tag_list.rb,
lib/chef/knife/bootstrap.rb,
lib/chef/knife/configure.rb,
lib/chef/knife/node_edit.rb,
lib/chef/knife/node_list.rb,
lib/chef/knife/node_show.rb,
lib/chef/knife/role_edit.rb,
lib/chef/knife/role_list.rb,
lib/chef/knife/role_show.rb,
lib/chef/knife/ssl_check.rb,
lib/chef/knife/ssl_fetch.rb,
lib/chef/knife/user_edit.rb,
lib/chef/knife/user_list.rb,
lib/chef/knife/user_show.rb,
lib/chef/knife/tag_create.rb,
lib/chef/knife/tag_delete.rb,
lib/chef/knife/client_edit.rb,
lib/chef/knife/client_list.rb,
lib/chef/knife/client_show.rb,
lib/chef/knife/node_create.rb,
lib/chef/knife/node_delete.rb,
lib/chef/knife/role_create.rb,
lib/chef/knife/role_delete.rb,
lib/chef/knife/user_create.rb,
lib/chef/knife/user_delete.rb,
lib/chef/knife/client_create.rb,
lib/chef/knife/client_delete.rb,
lib/chef/knife/cookbook_list.rb,
lib/chef/knife/cookbook_show.rb,
lib/chef/knife/cookbook_test.rb,
lib/chef/knife/data_bag_edit.rb,
lib/chef/knife/data_bag_list.rb,
lib/chef/knife/data_bag_show.rb,
lib/chef/knife/index_rebuild.rb,
lib/chef/knife/node_from_file.rb,
lib/chef/knife/role_from_file.rb,
lib/chef/knife/cookbook_create.rb,
lib/chef/knife/cookbook_delete.rb,
lib/chef/knife/cookbook_upload.rb,
lib/chef/knife/data_bag_create.rb,
lib/chef/knife/data_bag_delete.rb,
lib/chef/knife/user_reregister.rb,
lib/chef/knife/configure_client.rb,
lib/chef/knife/core/node_editor.rb,
lib/chef/knife/environment_edit.rb,
lib/chef/knife/environment_list.rb,
lib/chef/knife/environment_show.rb,
lib/chef/knife/node_bulk_delete.rb,
lib/chef/knife/role_bulk_delete.rb,
lib/chef/knife/client_reregister.rb,
lib/chef/knife/cookbook_download.rb,
lib/chef/knife/cookbook_metadata.rb,
lib/chef/knife/node_run_list_add.rb,
lib/chef/knife/node_run_list_set.rb,
lib/chef/knife/client_bulk_delete.rb,
lib/chef/knife/cookbook_site_list.rb,
lib/chef/knife/cookbook_site_show.rb,
lib/chef/knife/core/object_loader.rb,
lib/chef/knife/data_bag_from_file.rb,
lib/chef/knife/environment_create.rb,
lib/chef/knife/environment_delete.rb,
lib/chef/knife/cookbook_site_share.rb,
lib/chef/knife/core/node_presenter.rb,
lib/chef/knife/core/text_formatter.rb,
lib/chef/knife/environment_compare.rb,
lib/chef/knife/cookbook_bulk_delete.rb,
lib/chef/knife/cookbook_site_search.rb,
lib/chef/knife/node_run_list_remove.rb,
lib/chef/knife/cookbook_site_install.rb,
lib/chef/knife/cookbook_site_unshare.rb,
lib/chef/knife/environment_from_file.rb,
lib/chef/knife/cookbook_site_download.rb,
lib/chef/knife/core/bootstrap_context.rb,
lib/chef/knife/core/cookbook_scm_repo.rb,
lib/chef/knife/core/generic_presenter.rb,
lib/chef/knife/core/subcommand_loader.rb,
lib/chef/knife/cookbook_metadata_from_file.rb
Direct Known Subclasses
ChefFS::Knife, Bootstrap, ClientBulkDelete, ClientCreate, ClientDelete, ClientEdit, ClientList, ClientReregister, ClientShow, Configure, ConfigureClient, CookbookBulkDelete, CookbookCreate, CookbookDelete, CookbookDownload, CookbookList, CookbookMetadata, CookbookMetadataFromFile, CookbookShow, CookbookSiteDownload, CookbookSiteInstall, CookbookSiteList, CookbookSiteSearch, CookbookSiteShare, CookbookSiteShow, CookbookSiteUnshare, CookbookTest, CookbookUpload, DataBagCreate, DataBagDelete, DataBagEdit, DataBagFromFile, DataBagList, DataBagShow, EnvironmentCompare, EnvironmentCreate, EnvironmentDelete, EnvironmentEdit, EnvironmentFromFile, EnvironmentList, EnvironmentShow, Exec, Help, IndexRebuild, NodeBulkDelete, NodeCreate, NodeDelete, NodeEdit, NodeFromFile, NodeList, NodeRunListAdd, NodeRunListRemove, NodeRunListSet, NodeShow, Raw, RecipeList, RoleBulkDelete, RoleCreate, RoleDelete, RoleEdit, RoleFromFile, RoleList, RoleShow, Search, Ssh, SslCheck, SslFetch, Status, TagCreate, TagDelete, TagList, UserCreate, UserDelete, UserEdit, UserList, UserReregister, UserShow
Defined Under Namespace
Modules: Core
Classes: Bootstrap, ClientBulkDelete, ClientCreate, ClientDelete, ClientEdit, ClientList, ClientReregister, ClientShow, Configure, ConfigureClient, CookbookBulkDelete, CookbookCreate, CookbookDelete, CookbookDownload, CookbookList, CookbookMetadata, CookbookMetadataFromFile, CookbookSCMRepo, CookbookShow, CookbookSiteDownload, CookbookSiteInstall, CookbookSiteList, CookbookSiteSearch, CookbookSiteShare, CookbookSiteShow, CookbookSiteUnshare, CookbookSiteVendor, CookbookTest, CookbookUpload, DataBagCreate, DataBagDelete, DataBagEdit, DataBagFromFile, DataBagList, DataBagShow, Delete, Deps, Diff, Download, Edit, EnvironmentCompare, EnvironmentCreate, EnvironmentDelete, EnvironmentEdit, EnvironmentFromFile, EnvironmentList, EnvironmentShow, Exec, Help, IndexRebuild, List, NodeBulkDelete, NodeCreate, NodeDelete, NodeEdit, NodeEditor, NodeFromFile, NodeList, NodeRunListAdd, NodeRunListRemove, NodeRunListSet, NodeShow, Raw, RecipeList, RoleBulkDelete, RoleCreate, RoleDelete, RoleEdit, RoleFromFile, RoleList, RoleShow, Search, Show, Ssh, SslCheck, SslFetch, Status, SubcommandLoader, TagCreate, TagDelete, TagList, UI, Upload, UserCreate, UserDelete, UserEdit, UserList, UserReregister, UserShow, Xargs
Instance Attribute Summary collapse
Class Method Summary
collapse
Instance Method Summary
collapse
convert_to_class_name, convert_to_snake_case, filename_to_qualified_string, snake_case_basename
#enforce_path_sanity
Constructor Details
#initialize(argv = []) ⇒ Knife
Create a new instance of the current class configured for the given arguments and options
279
280
281
282
283
284
285
286
287
288
289
290
291
292
293
294
295
296
297
298
299
300
301
302
303
|
# File 'lib/chef/knife.rb', line 279
def initialize(argv=[])
super()
@ui = Chef::Knife::UI.new(STDOUT, STDERR, STDIN, config)
command_name_words = self.class.snake_case_name.split('_')
@name_args = parse_options(argv)
@name_args.delete(command_name_words.join('-'))
@name_args.reject! { |name_arg| command_name_words.delete(name_arg) }
command_name_words = command_name_words.join('_')
@name_args.reject! { |name_arg| command_name_words == name_arg }
if config[:help]
msg opt_parser
exit 1
end
Chef::Config[:verbosity] = config[:verbosity]
end
|
Instance Attribute Details
#name_args ⇒ Object
Returns the value of attribute name_args.
58
59
60
|
# File 'lib/chef/knife.rb', line 58
def name_args
@name_args
end
|
Returns the value of attribute ui.
59
60
61
|
# File 'lib/chef/knife.rb', line 59
def ui
@ui
end
|
Class Method Details
.category(new_category) ⇒ Object
Explicitly set the category for the current command to new_category The category is normally determined from the first word of the command name, but some commands make more sense using two or more words
Arguments
- new_category:
-
A String to set the category to (see examples)
Examples:
Data bag commands would be in the ‘data’ category by default. To put them in the ‘data bag’ category:
category('data bag')
94
95
96
|
# File 'lib/chef/knife.rb', line 94
def self.category(new_category)
@category = new_category
end
|
.common_name ⇒ Object
106
107
108
|
# File 'lib/chef/knife.rb', line 106
def self.common_name
snake_case_name.split('_').join(' ')
end
|
.config_fetcher(candidate_config) ⇒ Object
324
325
326
|
# File 'lib/chef/knife.rb', line 324
def self.config_fetcher(candidate_config)
Chef::ConfigFetcher.new(candidate_config, Chef::Config.config_file_jail)
end
|
.dependency_loaders ⇒ Object
205
206
207
|
# File 'lib/chef/knife.rb', line 205
def self.dependency_loaders
@dependency_loaders ||= []
end
|
.deps(&block) ⇒ Object
209
210
211
|
# File 'lib/chef/knife.rb', line 209
def self.deps(&block)
dependency_loaders << block
end
|
.guess_category(args) ⇒ Object
177
178
179
180
181
182
183
184
185
186
187
|
# File 'lib/chef/knife.rb', line 177
def self.guess_category(args)
category_words = args.select {|arg| arg =~ /^(([[:alnum:]])[[:alnum:]\_\-]+)$/ }
category_words.map! {|w| w.split('-')}.flatten!
matching_category = nil
while (!matching_category) && (!category_words.empty?)
candidate_category = category_words.join(' ')
matching_category = candidate_category if subcommands_by_category.key?(candidate_category)
matching_category || category_words.pop
end
matching_category
end
|
.inherited(subclass) ⇒ Object
79
80
81
82
83
|
# File 'lib/chef/knife.rb', line 79
def self.inherited(subclass)
unless subclass.unnamed?
subcommands[subclass.snake_case_name] = subclass
end
end
|
.list_commands(preferred_category = nil) ⇒ Object
Print the list of subcommands knife knows about. If preferred_category is given, only subcommands in that category are shown
139
140
141
142
143
144
145
146
147
148
149
150
151
152
153
154
155
156
157
158
159
|
# File 'lib/chef/knife.rb', line 139
def self.list_commands(preferred_category=nil)
load_commands
category_desc = preferred_category ? preferred_category + " " : ''
msg "Available #{category_desc}subcommands: (for details, knife SUB-COMMAND --help)\n\n"
if preferred_category && subcommands_by_category.key?(preferred_category)
commands_to_show = {preferred_category => subcommands_by_category[preferred_category]}
else
commands_to_show = subcommands_by_category
end
commands_to_show.sort.each do |category, commands|
next if category =~ /deprecated/i
msg "** #{category.upcase} COMMANDS **"
commands.sort.each do |command|
msg subcommands[command].banner if subcommands[command]
end
msg
end
end
|
.load_commands ⇒ Object
119
120
121
|
# File 'lib/chef/knife.rb', line 119
def self.load_commands
@commands_loaded ||= subcommand_loader.load_commands
end
|
.load_deps ⇒ Object
213
214
215
216
217
|
# File 'lib/chef/knife.rb', line 213
def self.load_deps
dependency_loaders.each do |dep_loader|
dep_loader.call
end
end
|
.locate_config_file ⇒ Object
328
329
330
331
332
333
334
335
336
337
338
339
340
341
342
343
344
345
346
347
348
349
350
351
352
353
354
355
|
# File 'lib/chef/knife.rb', line 328
def self.locate_config_file
candidate_configs = []
if ENV['KNIFE_HOME']
candidate_configs << File.join(ENV['KNIFE_HOME'], 'knife.rb')
end
if Dir.pwd
candidate_configs << File.join(Dir.pwd, 'knife.rb')
end
if chef_config_dir
candidate_configs << File.join(chef_config_dir, 'knife.rb')
end
if ENV['HOME']
candidate_configs << File.join(ENV['HOME'], '.chef', 'knife.rb')
end
candidate_configs.each do | candidate_config |
fetcher = config_fetcher(candidate_config)
if !fetcher.config_missing?
return candidate_config
end
end
return nil
end
|
.msg(msg = "") ⇒ Object
70
71
72
|
# File 'lib/chef/knife.rb', line 70
def self.msg(msg="")
ui.msg(msg)
end
|
.reset_subcommands! ⇒ Object
74
75
76
77
|
# File 'lib/chef/knife.rb', line 74
def self.reset_subcommands!
@@subcommands = {}
@subcommands_by_category = nil
end
|
.run(args, options = {}) ⇒ Object
Run knife for the given args (ARGV), adding options to the list of CLI options that the subcommand knows how to handle.
Arguments
- args:
-
usually ARGV
- options:
-
A Mixlib::CLI option parser hash. These options are how
subcommands know about global knife CLI options
167
168
169
170
171
172
173
174
175
|
# File 'lib/chef/knife.rb', line 167
def self.run(args, options={})
load_commands
subcommand_class = subcommand_class_from(args)
subcommand_class.options = options.merge!(subcommand_class.options)
subcommand_class.load_deps
instance = subcommand_class.new(args)
instance.configure_chef
instance.run_with_pretty_exceptions
end
|
.snake_case_name ⇒ Object
102
103
104
|
# File 'lib/chef/knife.rb', line 102
def self.snake_case_name
convert_to_snake_case(name.split('::').last) unless unnamed?
end
|
.subcommand_category ⇒ Object
98
99
100
|
# File 'lib/chef/knife.rb', line 98
def self.subcommand_category
@category || snake_case_name.split('_').first unless unnamed?
end
|
.subcommand_class_from(args) ⇒ Object
189
190
191
192
193
194
195
196
197
198
199
200
201
202
203
|
# File 'lib/chef/knife.rb', line 189
def self.subcommand_class_from(args)
command_words = args.select {|arg| arg =~ /^(([[:alnum:]])[[:alnum:]\_\-]+)$/ }
subcommand_class = nil
while ( !subcommand_class ) && ( !command_words.empty? )
snake_case_class_name = command_words.join("_")
unless subcommand_class = subcommands[snake_case_class_name]
command_words.pop
end
end
subcommand_class ||= subcommands[args.first.gsub('-', '_')]
subcommand_class || subcommand_not_found!(args)
end
|
.subcommand_loader ⇒ Object
115
116
117
|
# File 'lib/chef/knife.rb', line 115
def self.subcommand_loader
@subcommand_loader ||= Knife::SubcommandLoader.new(chef_config_dir)
end
|
.subcommands ⇒ Object
123
124
125
|
# File 'lib/chef/knife.rb', line 123
def self.subcommands
@@subcommands ||= {}
end
|
.subcommands_by_category ⇒ Object
127
128
129
130
131
132
133
134
135
|
# File 'lib/chef/knife.rb', line 127
def self.subcommands_by_category
unless @subcommands_by_category
@subcommands_by_category = Hash.new { |hash, key| hash[key] = [] }
subcommands.each do |snake_cased, klass|
@subcommands_by_category[klass.subcommand_category] << snake_cased
end
end
@subcommands_by_category
end
|
66
67
68
|
# File 'lib/chef/knife.rb', line 66
def self.ui
@ui ||= Chef::Knife::UI.new(STDOUT, STDERR, STDIN, {})
end
|
.unnamed? ⇒ Boolean
Does this class have a name? (Classes created via Class.new don’t)
111
112
113
|
# File 'lib/chef/knife.rb', line 111
def self.unnamed?
name.nil? || name.empty?
end
|
.use_separate_defaults? ⇒ Boolean
Configure mixlib-cli to always separate defaults from user-supplied CLI options
62
63
64
|
# File 'lib/chef/knife.rb', line 62
def self.use_separate_defaults?
true
end
|
Instance Method Details
561
562
563
|
# File 'lib/chef/knife.rb', line 561
def api_key
Chef::Config[:client_key]
end
|
#apply_computed_config ⇒ Object
Catch-all method that does any massaging needed for various config components, such as expanding file paths and converting verbosity level into log level.
376
377
378
379
380
381
382
383
384
385
386
387
388
389
390
391
392
393
394
395
396
397
398
399
400
401
402
403
404
405
406
407
408
409
410
411
412
413
414
|
# File 'lib/chef/knife.rb', line 376
def apply_computed_config
Chef::Config[:color] = config[:color]
case Chef::Config[:verbosity]
when 0, nil
Chef::Config[:log_level] = :error
when 1
Chef::Config[:log_level] = :info
else
Chef::Config[:log_level] = :debug
end
Chef::Config[:node_name] = config[:node_name] if config[:node_name]
Chef::Config[:client_key] = config[:client_key] if config[:client_key]
Chef::Config[:chef_server_url] = config[:chef_server_url] if config[:chef_server_url]
Chef::Config[:environment] = config[:environment] if config[:environment]
Chef::Config.local_mode = config[:local_mode] if config.has_key?(:local_mode)
if Chef::Config.local_mode && !Chef::Config.has_key?(:cookbook_path) && !Chef::Config.has_key?(:chef_repo_path)
Chef::Config.chef_repo_path = Chef::Config.find_chef_repo_path(Dir.pwd)
end
Chef::Config.chef_zero.port = config[:chef_zero_port] if config[:chef_zero_port]
if Chef::Config[:client_key] && config[:config_file]
Chef::Config[:client_key] = File.expand_path(Chef::Config[:client_key], File.dirname(config[:config_file]))
end
Mixlib::Log::Formatter.show_time = false
Chef::Log.init(Chef::Config[:log_location])
Chef::Log.level(Chef::Config[:log_level] || :error)
if Chef::Config[:node_name] && Chef::Config[:node_name].bytesize > 90
Chef::Config[:authentication_protocol_version] = "1.1"
end
end
|
#config_file_settings ⇒ Object
Returns a subset of the Chef::Config Hash that is relevant to the currently executing knife command. This is used by #configure_chef to apply settings from knife.rb to the config hash.
316
317
318
319
320
321
322
|
# File 'lib/chef/knife.rb', line 316
def config_file_settings
config_file_settings = {}
self.class.options.keys.each do |key|
config_file_settings[key] = Chef::Config[:knife][key] if Chef::Config[:knife].has_key?(key)
end
config_file_settings
end
|
416
417
418
419
420
421
422
423
424
425
426
427
428
429
430
431
432
433
434
435
436
437
438
439
440
|
# File 'lib/chef/knife.rb', line 416
def configure_chef
if !config[:config_file]
located_config_file = self.class.locate_config_file
config[:config_file] = located_config_file if located_config_file
end
if config[:config_file]
Chef::Config.config_file = config[:config_file]
fetcher = Chef::ConfigFetcher.new(config[:config_file], Chef::Config.config_file_jail)
if fetcher.config_missing?
ui.error("Specified config file #{config[:config_file]} does not exist#{Chef::Config.config_file_jail ? " or is not under config file jail #{Chef::Config.config_file_jail}" : ""}!")
exit 1
end
Chef::Log.debug("Using configuration from #{config[:config_file]}")
read_config(fetcher.read_config, config[:config_file])
else
Chef::Config[:color] = config[:color]
ui.warn("No knife configuration file found")
end
merge_configs
apply_computed_config
end
|
#create_object(object, pretty_name = nil, &block) ⇒ Object
575
576
577
578
579
580
581
582
583
584
585
586
587
588
589
|
# File 'lib/chef/knife.rb', line 575
def create_object(object, pretty_name=nil, &block)
output = edit_data(object)
if Kernel.block_given?
output = block.call(output)
else
output.save
end
pretty_name ||= output
self.msg("Created #{pretty_name}")
output(output) if config[:print_after]
end
|
#delete_object(klass, name, delete_name = nil, &block) ⇒ Object
591
592
593
594
595
596
597
598
599
600
601
602
603
604
605
|
# File 'lib/chef/knife.rb', line 591
def delete_object(klass, name, delete_name=nil, &block)
confirm("Do you really want to delete #{name}")
if Kernel.block_given?
object = block.call
else
object = klass.load(name)
object.destroy
end
output(format_for_display(object)) if config[:print_after]
obj_name = delete_name ? "#{delete_name}[#{name}]" : object
self.msg("Deleted #{obj_name}")
end
|
Parses JSON from the error response sent by Chef Server and returns the error message – TODO: this code belongs in Chef::REST
569
570
571
572
573
|
# File 'lib/chef/knife.rb', line 569
def format_rest_error(response)
Array(Chef::JSONCompat.from_json(response.body)["error"]).join('; ')
rescue Exception
response.body
end
|
#highlight_config_error(file, line) ⇒ Object
465
466
467
468
469
470
471
472
473
474
475
476
477
478
479
|
# File 'lib/chef/knife.rb', line 465
def highlight_config_error(file, line)
config_file_lines = []
IO.readlines(file).each_with_index {|l, i| config_file_lines << "#{(i + 1).to_s.rjust(3)}: #{l.chomp}"}
if line == 1
lines = config_file_lines[0..3]
lines[0] = ui.color(lines[0], :red)
else
lines = config_file_lines[Range.new(line - 2, line)]
lines[1] = ui.color(lines[1], :red)
end
ui.msg ""
ui.msg ui.color(" # #{file}", :white)
lines.each {|l| ui.msg(l)}
ui.msg ""
end
|
#humanize_exception(e) ⇒ Object
502
503
504
505
506
507
508
509
510
511
512
513
514
515
516
517
518
519
520
521
522
523
524
525
|
# File 'lib/chef/knife.rb', line 502
def humanize_exception(e)
case e
when SystemExit
raise
when Net::HTTPServerException, Net::HTTPFatalError
humanize_http_exception(e)
when Errno::ECONNREFUSED, Timeout::Error, Errno::ETIMEDOUT, SocketError
ui.error "Network Error: #{e.message}"
ui.info "Check your knife configuration and network settings"
when NameError, NoMethodError
ui.error "knife encountered an unexpected error"
ui.info "This may be a bug in the '#{self.class.common_name}' knife command or plugin"
ui.info "Please collect the output of this command with the `-VV` option before filing a bug report."
ui.info "Exception: #{e.class.name}: #{e.message}"
when Chef::Exceptions::PrivateKeyMissing
ui.error "Your private key could not be loaded from #{api_key}"
ui.info "Check your configuration file and ensure that your private key is readable"
when Chef::Exceptions::InvalidRedirect
ui.error "Invalid Redirect: #{e.message}"
ui.info "Change your server location in knife.rb to the server's FQDN to avoid unwanted redirections."
else
ui.error "#{e.class.name}: #{e.message}"
end
end
|
#humanize_http_exception(e) ⇒ Object
527
528
529
530
531
532
533
534
535
536
537
538
539
540
541
542
543
544
545
546
547
548
549
550
551
552
553
554
555
|
# File 'lib/chef/knife.rb', line 527
def humanize_http_exception(e)
response = e.response
case response
when Net::HTTPUnauthorized
ui.error "Failed to authenticate to #{server_url} as #{username} with key #{api_key}"
ui.info "Response: #{format_rest_error(response)}"
when Net::HTTPForbidden
ui.error "You authenticated successfully to #{server_url} as #{username} but you are not authorized for this action"
ui.info "Response: #{format_rest_error(response)}"
when Net::HTTPBadRequest
ui.error "The data in your request was invalid"
ui.info "Response: #{format_rest_error(response)}"
when Net::HTTPNotFound
ui.error "The object you are looking for could not be found"
ui.info "Response: #{format_rest_error(response)}"
when Net::HTTPInternalServerError
ui.error "internal server error"
ui.info "Response: #{format_rest_error(response)}"
when Net::HTTPBadGateway
ui.error "bad gateway"
ui.info "Response: #{format_rest_error(response)}"
when Net::HTTPServiceUnavailable
ui.error "Service temporarily unavailable"
ui.info "Response: #{format_rest_error(response)}"
else
ui.error response.message
ui.info "Response: #{format_rest_error(response)}"
end
end
|
#merge_configs ⇒ Object
Apply Config in this order: defaults from mixlib-cli settings from config file, via Chef::Config config from command line
361
362
363
364
365
366
367
368
369
370
371
|
# File 'lib/chef/knife.rb', line 361
def merge_configs
combined_config = default_config.merge(config_file_settings)
combined_config = combined_config.merge(config)
config.replace(combined_config)
end
|
#noauth_rest ⇒ Object
614
615
616
617
618
619
|
# File 'lib/chef/knife.rb', line 614
def noauth_rest
@rest ||= begin
require 'chef/rest'
Chef::REST.new(Chef::Config[:chef_server_url], false, false)
end
end
|
#parse_options(args) ⇒ Object
305
306
307
308
309
310
311
|
# File 'lib/chef/knife.rb', line 305
def parse_options(args)
super
rescue OptionParser::InvalidOption => e
puts "Error: " + e.to_s
show_usage
exit(1)
end
|
#read_config(config_content, config_file_path) ⇒ Object
442
443
444
445
446
447
448
449
450
451
452
453
454
455
456
457
458
459
460
461
462
463
|
# File 'lib/chef/knife.rb', line 442
def read_config(config_content, config_file_path)
Chef::Config.from_string(config_content, config_file_path)
rescue SyntaxError => e
ui.error "You have invalid ruby syntax in your config file #{config_file_path}"
ui.info(ui.color(e.message, :red))
if file_line = e.message[/#{Regexp.escape(config_file_path)}:[\d]+/]
line = file_line[/:([\d]+)$/, 1].to_i
highlight_config_error(config_file_path, line)
end
exit 1
rescue Exception => e
ui.error "You have an error in your config file #{config_file_path}"
ui.info "#{e.class.name}: #{e.message}"
filtered_trace = e.backtrace.grep(/#{Regexp.escape(config_file_path)}/)
filtered_trace.each {|line| ui.msg(" " + ui.color(line, :red))}
if !filtered_trace.empty?
line_nr = filtered_trace.first[/#{Regexp.escape(config_file_path)}:([\d]+)/, 1]
highlight_config_error(config_file_path, line_nr.to_i)
end
exit 1
end
|
607
608
609
610
611
612
|
# File 'lib/chef/knife.rb', line 607
def rest
@rest ||= begin
require 'chef/rest'
Chef::REST.new(Chef::Config[:chef_server_url])
end
end
|
#run_with_pretty_exceptions(raise_exception = false) ⇒ Object
485
486
487
488
489
490
491
492
493
494
495
496
497
498
499
500
|
# File 'lib/chef/knife.rb', line 485
def run_with_pretty_exceptions(raise_exception = false)
unless self.respond_to?(:run)
ui.error "You need to add a #run method to your knife command before you can use it"
end
enforce_path_sanity
Chef::Application.setup_server_connectivity
begin
run
ensure
Chef::Application.destroy_server_connectivity
end
rescue Exception => e
raise if raise_exception || Chef::Config[:verbosity] == 2
humanize_exception(e)
exit 100
end
|
#server_url ⇒ Object
621
622
623
|
# File 'lib/chef/knife.rb', line 621
def server_url
Chef::Config[:chef_server_url]
end
|
#show_usage ⇒ Object
481
482
483
|
# File 'lib/chef/knife.rb', line 481
def show_usage
stdout.puts("USAGE: " + self.opt_parser.to_s)
end
|
557
558
559
|
# File 'lib/chef/knife.rb', line 557
def username
Chef::Config[:node_name]
end
|