Class: Chef::Provider::Cron

Inherits:
Chef::Provider show all
Includes:
Mixin::Command
Defined in:
lib/chef/provider/cron.rb,
lib/chef/provider/cron/solaris.rb

Direct Known Subclasses

Solaris

Defined Under Namespace

Classes: Solaris

Constant Summary collapse

CRON_PATTERN =
/\A([-0-9*,\/]+)\s([-0-9*,\/]+)\s([-0-9*,\/]+)\s([-0-9*,\/]+)\s([-0-9*,\/]+)\s(.*)/
ENV_PATTERN =
/\A(\S+)=(\S*)/
CRON_ATTRIBUTES =
[:minute, :hour, :day, :month, :weekday, :command, :mailto, :path, :shell, :home, :environment]

Instance Attribute Summary collapse

Attributes inherited from Chef::Provider

#current_resource, #new_resource, #run_context

Instance Method Summary collapse

Methods included from Mixin::Command

#chdir_or_tmpdir, #handle_command_failures, #output_of_command, #run_command, #run_command_with_systems_locale

Methods included from Mixin::Command::Windows

#popen4

Methods included from Mixin::Command::Unix

#popen4

Methods inherited from Chef::Provider

#action_nothing, build_from_file, #cookbook_name, #node, #resource_collection

Methods included from Mixin::ConvertToClassName

#convert_to_class_name, #convert_to_snake_case, #filename_to_qualified_string, #snake_case_basename

Methods included from Mixin::EnforceOwnershipAndPermissions

#enforce_ownership_and_permissions

Methods included from Mixin::RecipeDefinitionDSLCore

#method_missing

Methods included from Mixin::Language

#data_bag, #data_bag_item, #platform?, #platform_family?, #search, #value_for_platform, #value_for_platform_family

Constructor Details

#initialize(new_resource, run_context) ⇒ Cron

Returns a new instance of Cron.



33
34
35
36
37
# File 'lib/chef/provider/cron.rb', line 33

def initialize(new_resource, run_context)
  super(new_resource, run_context)
  @cron_exists = false
  @cron_empty = false
end

Dynamic Method Handling

This class handles dynamic methods through the method_missing method in the class Chef::Mixin::RecipeDefinitionDSLCore

Instance Attribute Details

#cron_emptyObject

Returns the value of attribute cron_empty.



38
39
40
# File 'lib/chef/provider/cron.rb', line 38

def cron_empty
  @cron_empty
end

#cron_existsObject

Returns the value of attribute cron_exists.



38
39
40
# File 'lib/chef/provider/cron.rb', line 38

def cron_exists
  @cron_exists
end

Instance Method Details

#action_createObject



87
88
89
90
91
92
93
94
95
96
97
98
99
100
101
102
103
104
105
106
107
108
109
110
111
112
113
114
115
116
117
118
119
120
121
122
123
124
125
126
127
128
129
130
131
132
133
134
135
136
137
138
139
140
141
142
143
# File 'lib/chef/provider/cron.rb', line 87

def action_create
  crontab = String.new
  newcron = String.new
  cron_found = false

  newcron << "# Chef Name: #{new_resource.name}\n"
  [ :mailto, :path, :shell, :home ].each do |v|
    newcron << "#{v.to_s.upcase}=#{@new_resource.send(v)}\n" if @new_resource.send(v)
  end
  @new_resource.environment.each do |name, value|
    newcron << "#{name}=#{value}\n"
  end
  newcron << "#{@new_resource.minute} #{@new_resource.hour} #{@new_resource.day} #{@new_resource.month} #{@new_resource.weekday} #{@new_resource.command}\n"

  if @cron_exists
    unless cron_different?
      Chef::Log.debug("Skipping existing cron entry '#{@new_resource.name}'")
      return
    end
    read_crontab.each_line do |line|
      case line.chomp
      when "# Chef Name: #{@new_resource.name}"
        cron_found = true
        next
      when ENV_PATTERN
        crontab << line unless cron_found
        next
      when CRON_PATTERN
        if cron_found
          cron_found = false
          crontab << newcron
          next
        end
      else
        if cron_found # We've got a Chef comment with no following crontab line
          crontab << newcron
          cron_found = false
        end
      end
      crontab << line
    end
    # Handle edge case where the Chef comment is the last line in the current crontab
    crontab << newcron if cron_found

    write_crontab crontab
    Chef::Log.info("#{@new_resource} updated crontab entry")
    @new_resource.updated_by_last_action(true)
  else
    crontab = read_crontab unless @cron_empty

    crontab << newcron

    write_crontab crontab
    Chef::Log.info("#{@new_resource} added crontab entry")
    @new_resource.updated_by_last_action(true)
  end
end

#action_deleteObject



145
146
147
148
149
150
151
152
153
154
155
156
157
158
159
160
161
162
163
164
165
166
167
168
169
170
171
172
# File 'lib/chef/provider/cron.rb', line 145

def action_delete
  if @cron_exists
    crontab = String.new
    cron_found = false
    read_crontab.each_line do |line|
      case line.chomp
      when "# Chef Name: #{@new_resource.name}"
        cron_found = true
        next
      when ENV_PATTERN
        next if cron_found
      when CRON_PATTERN
        if cron_found
          cron_found = false
          next
        end
      else
        # We've got a Chef comment with no following crontab line
        cron_found = false
      end
      crontab << line
    end

    write_crontab crontab
    Chef::Log.info("#{@new_resource} deleted crontab entry")
    @new_resource.updated_by_last_action(true)
  end
end

#cron_different?Boolean

Returns:

  • (Boolean)


81
82
83
84
85
# File 'lib/chef/provider/cron.rb', line 81

def cron_different?
  CRON_ATTRIBUTES.any? do |cron_var|
    !@new_resource.send(cron_var).nil? && @new_resource.send(cron_var) != @current_resource.send(cron_var)
  end
end

#load_current_resourceObject



40
41
42
43
44
45
46
47
48
49
50
51
52
53
54
55
56
57
58
59
60
61
62
63
64
65
66
67
68
69
70
71
72
73
74
75
76
77
78
79
# File 'lib/chef/provider/cron.rb', line 40

def load_current_resource
  crontab_lines = []
  @current_resource = Chef::Resource::Cron.new(@new_resource.name)
  @current_resource.user(@new_resource.user)
  if crontab = read_crontab
    cron_found = false
    crontab.each_line do |line|
      case line.chomp
      when "# Chef Name: #{@new_resource.name}"
        Chef::Log.debug("Found cron '#{@new_resource.name}'")
        cron_found = true
        @cron_exists = true
        next
      when ENV_PATTERN
        set_environment_var($1, $2) if cron_found
        next
      when CRON_PATTERN
        if cron_found
          @current_resource.minute($1)
          @current_resource.hour($2)
          @current_resource.day($3)
          @current_resource.month($4)
          @current_resource.weekday($5)
          @current_resource.command($6)
          cron_found=false
        end
        next
      else
        cron_found=false # We've got a Chef comment with no following crontab line
        next
      end
    end
    Chef::Log.debug("Cron '#{@new_resource.name}' not found") unless @cron_exists
  else
    Chef::Log.debug("Cron empty for '#{@new_resource.user}'")
    @cron_empty = true
  end

  @current_resource
end