Class: Chef::Provider::Package::Dnf

Inherits:
Chef::Provider::Package show all
Extended by:
Mixin::ShellOut, Mixin::Which
Includes:
Mixin::GetSourceFromPackage
Defined in:
lib/chef/provider/package/dnf.rb,
lib/chef/provider/package/dnf/version.rb,
lib/chef/provider/package/dnf/python_helper.rb

Defined Under Namespace

Classes: PythonHelper, Version

Instance Attribute Summary

Attributes inherited from Chef::Provider

#action, #after_resource, #current_resource, #logger, #new_resource, #run_context

Instance Method Summary collapse

Methods included from Mixin::GetSourceFromPackage

#initialize

Methods inherited from Chef::Provider::Package

#as_array, #check_resource_semantics!, #expand_options, #have_any_matching_version?, #initialize, #multipackage_api_adapter, #options, #package_locked, #prepare_for_installation, #preseed_package, #reconfig_package, #removing_package?, #target_version_already_installed?, #version_requirement_satisfied?

Methods included from Mixin::SubclassDirective

#subclass_directive

Methods inherited from Chef::Provider

action, action_description, action_descriptions, #action_nothing, #check_resource_semantics!, #cleanup_after_converge, #compile_and_converge_action, #converge_by, #converge_if_changed, #cookbook_name, #description, #events, include_resource_dsl?, include_resource_dsl_module, #initialize, #introduced, #node, #process_resource_requirements, provides, provides?, #recipe_name, #requirements, #resource_collection, #resource_updated?, #run_action, #set_updated_status, supports?, use, use_inline_resources, #validate_required_properties!, #whyrun_mode?, #whyrun_supported?

Methods included from Mixin::Provides

#provided_as, #provides, #provides?

Methods included from Mixin::DescendantsTracker

descendants, #descendants, direct_descendants, #direct_descendants, find_descendants_by_name, #find_descendants_by_name, #inherited, store_inherited

Methods included from Mixin::LazyModuleInclude

#descendants, #include, #included

Methods included from Mixin::PowershellOut

#powershell_out, #powershell_out!

Methods included from Mixin::WindowsArchitectureHelper

#assert_valid_windows_architecture!, #disable_wow64_file_redirection, #forced_32bit_override_required?, #is_i386_process_on_x86_64_windows?, #node_supports_windows_architecture?, #node_windows_architecture, #restore_wow64_file_redirection, #valid_windows_architecture?, #with_os_architecture, #wow64_architecture_override_required?, #wow64_directory

Methods included from DSL::Secret

#default_secret_config, #default_secret_service, #secret, #with_secret_config, #with_secret_service

Methods included from DSL::RenderHelpers

#render_json, #render_toml, #render_yaml

Methods included from DSL::ReaderHelpers

#parse_file, #parse_json, #parse_toml, #parse_yaml

Methods included from DSL::Powershell

#ps_credential

Methods included from DSL::RegistryHelper

#registry_data_exists?, #registry_get_subkeys, #registry_get_values, #registry_has_subkeys?, #registry_key_exists?, #registry_value_exists?

Methods included from DSL::ChefVault

#chef_vault, #chef_vault_item, #chef_vault_item_for_environment

Methods included from DSL::DataQuery

#data_bag, #data_bag_item, #search, #tagged?

Methods included from EncryptedDataBagItem::CheckEncrypted

#encrypted?

Methods included from DSL::PlatformIntrospection

#older_than_win_2012_or_8?, #platform?, #platform_family?, #value_for_platform, #value_for_platform_family

Methods included from DSL::Recipe

#exec, #have_resource_class_for?, #resource_class_for

Methods included from DSL::Definitions

add_definition, #evaluate_resource_definition, #has_resource_definition?

Methods included from DSL::Resources

add_resource_dsl, remove_resource_dsl

Methods included from DSL::Cheffish

load_cheffish

Methods included from DSL::RebootPending

#reboot_pending?

Methods included from DSL::IncludeRecipe

#include_recipe, #load_recipe

Methods included from Mixin::NotifyingBlock

#notifying_block, #subcontext_block

Methods included from DSL::DeclareResource

#build_resource, #declare_resource, #delete_resource, #delete_resource!, #edit_resource, #edit_resource!, #find_resource, #find_resource!, #resources, #with_run_context

Methods included from DSL::Compliance

#include_input, #include_profile, #include_waiver

Instance Method Details

#candidate_versionObject



104
105
106
107
108
# File 'lib/chef/provider/package/dnf.rb', line 104

def candidate_version
  package_name_array.each_with_index.map do |pkg, i|
    available_version(i).version_with_arch
  end
end

#define_resource_requirementsObject



94
95
96
97
98
99
100
101
102
# File 'lib/chef/provider/package/dnf.rb', line 94

def define_resource_requirements
  super

  requirements.assert(:install, :upgrade, :remove, :purge) do |a|
    a.assertion { !new_resource.source || ::File.exist?(new_resource.source) }
    a.failure_message Chef::Exceptions::Package, "Package #{new_resource.package_name} not found: #{new_resource.source}"
    a.whyrun "assuming #{new_resource.source} would have previously been created"
  end
end

#dnf5?Boolean

Returns:

  • (Boolean)


67
68
69
70
71
72
# File 'lib/chef/provider/package/dnf.rb', line 67

def dnf5?
  @dnf5 ||= begin
    dnf_version = shell_out!("dnf --version").stdout
    dnf_version =~ /dnf5/i
  end
end

#get_current_versionsObject



116
117
118
119
120
# File 'lib/chef/provider/package/dnf.rb', line 116

def get_current_versions
  package_name_array.each_with_index.map do |pkg, i|
    current_version(i).version_with_arch
  end
end

#install_package(names, versions) ⇒ Object Also known as: upgrade_package



122
123
124
125
126
127
128
129
130
# File 'lib/chef/provider/package/dnf.rb', line 122

def install_package(names, versions)
  if new_resource.source
    dnf(options, "-y", "install", new_resource.source)
  else
    resolved_names = names.each_with_index.map { |name, i| available_version(i).to_s unless name.nil? }
    dnf(options, "-y", "install", resolved_names)
  end
  flushcache
end

#load_after_resourceObject



84
85
86
87
88
89
90
91
92
# File 'lib/chef/provider/package/dnf.rb', line 84

def load_after_resource
  # force the installed version array to repopulate
  @current_version = []
  @after_resource = Chef::Resource::DnfPackage.new(new_resource.name)
  after_resource.package_name(new_resource.package_name)
  after_resource.version(get_current_versions)

  after_resource
end

#load_current_resourceObject



74
75
76
77
78
79
80
81
82
# File 'lib/chef/provider/package/dnf.rb', line 74

def load_current_resource
  flushcache if new_resource.flush_cache[:before]

  @current_resource = Chef::Resource::DnfPackage.new(new_resource.name)
  current_resource.package_name(new_resource.package_name)
  current_resource.version(get_current_versions)

  current_resource
end

#lock_package(names, versions) ⇒ Object

NB: the dnf_package provider manages individual single packages, please do not submit issues or PRs to try to add wildcard support to lock / unlock. The best solution is to write an execute resource which does a not_if ‘dnf versionlock | grep ’^pattern“ kind of approach



149
150
151
152
# File 'lib/chef/provider/package/dnf.rb', line 149

def lock_package(names, versions)
  default_opts = dnf5? ? [] : %w{-d0 -e0}
  dnf(default_opts, options, "versionlock", "add", resolved_package_lock_names(names))
end

#magic_version_arrayObject



110
111
112
113
114
# File 'lib/chef/provider/package/dnf.rb', line 110

def magic_version_array
  package_name_array.each_with_index.map do |pkg, i|
    magical_version(i).version_with_arch
  end
end

#python_helperObject

Most of the magic in this class happens in the python helper script. The ruby side of this provider knows only enough to translate Chef-style new_resource name+package+version into a request to the python side. The python side is then responsible for knowing everything about RPMs and what is installed and what is available. The ruby side of this class should remain a lightweight translation layer to translate Chef requests into RPC requests to python. This class knows nothing about how to compare RPM versions, and does not maintain any cached state of installed/available versions and should be kept that way.



63
64
65
# File 'lib/chef/provider/package/dnf.rb', line 63

def python_helper
  @python_helper ||= PythonHelper.instance
end

#remove_package(names, versions) ⇒ Object Also known as: purge_package



135
136
137
138
139
# File 'lib/chef/provider/package/dnf.rb', line 135

def remove_package(names, versions)
  resolved_names = names.each_with_index.map { |name, i| magical_version(i).to_s unless name.nil? }
  dnf(options, "-y", "remove", resolved_names)
  flushcache
end

#unlock_package(names, versions) ⇒ Object

NB: the dnf_package provider manages individual single packages, please do not submit issues or PRs to try to add wildcard support to lock / unlock. The best solution is to write an execute resource which does a only_if ‘dnf versionlock | grep ’^pattern“ kind of approach



156
157
158
159
160
161
162
163
# File 'lib/chef/provider/package/dnf.rb', line 156

def unlock_package(names, versions)
  if dnf5?
    dnf("-y", options, "versionlock", "delete", resolved_package_lock_names(names))
  else
    # dnf versionlock delete on rhel6 needs the glob nonsense in the following command
    dnf("-d0", "-e0", "-y", options, "versionlock", "delete", resolved_package_lock_names(names).map { |n| "*:#{n}-*" })
  end
end