Class: Chef::ReservedNames::Win32::Security::SID

Inherits:
Object
  • Object
show all
Extended by:
API::Error, API::Net
Includes:
API::Error, API::Net
Defined in:
lib/chef/win32/security/sid.rb

Constant Summary

Constants included from API::Error

API::Error::DONT_RESOLVE_DLL_REFERENCES, API::Error::EPT_S_CANT_CREATE, API::Error::EPT_S_CANT_PERFORM_OP, API::Error::EPT_S_INVALID_ENTRY, API::Error::EPT_S_NOT_REGISTERED, API::Error::ERROR_ACCESS_DENIED, API::Error::ERROR_ACCESS_DISABLED_BY_POLICY, API::Error::ERROR_ACCOUNT_DISABLED, API::Error::ERROR_ACCOUNT_EXPIRED, API::Error::ERROR_ACCOUNT_LOCKED_OUT, API::Error::ERROR_ACCOUNT_RESTRICTION, API::Error::ERROR_ACTIVE_CONNECTIONS, API::Error::ERROR_ADAP_HDW_ERR, API::Error::ERROR_ADDRESS_ALREADY_ASSOCIATED, API::Error::ERROR_ADDRESS_NOT_ASSOCIATED, API::Error::ERROR_ALIAS_EXISTS, API::Error::ERROR_ALLOTTED_SPACE_EXCEEDED, API::Error::ERROR_ALL_USER_TRUST_QUOTA_EXCEEDED, API::Error::ERROR_ALREADY_ASSIGNED, API::Error::ERROR_ALREADY_EXISTS, API::Error::ERROR_ALREADY_INITIALIZED, API::Error::ERROR_ALREADY_REGISTERED, API::Error::ERROR_ALREADY_RUNNING_LKG, API::Error::ERROR_ALREADY_WAITING, API::Error::ERROR_APP_WRONG_OS, API::Error::ERROR_ARENA_TRASHED, API::Error::ERROR_ARITHMETIC_OVERFLOW, API::Error::ERROR_ATOMIC_LOCKS_NOT_SUPPORTED, API::Error::ERROR_AUTHENTICATION_FIREWALL_FAILED, API::Error::ERROR_AUTODATASEG_EXCEEDS_64k, API::Error::ERROR_BADDB, API::Error::ERROR_BADKEY, API::Error::ERROR_BAD_ARGUMENTS, API::Error::ERROR_BAD_COMMAND, API::Error::ERROR_BAD_CONFIGURATION, API::Error::ERROR_BAD_DESCRIPTOR_FORMAT, API::Error::ERROR_BAD_DEVICE, API::Error::ERROR_BAD_DEV_TYPE, API::Error::ERROR_BAD_DRIVER, API::Error::ERROR_BAD_DRIVER_LEVEL, API::Error::ERROR_BAD_DYNALINK, API::Error::ERROR_BAD_ENVIRONMENT, API::Error::ERROR_BAD_EXE_FORMAT, API::Error::ERROR_BAD_FORMAT, API::Error::ERROR_BAD_IMPERSONATION_LEVEL, API::Error::ERROR_BAD_INHERITANCE_ACL, API::Error::ERROR_BAD_LENGTH, API::Error::ERROR_BAD_LOGON_SESSION_STATE, API::Error::ERROR_BAD_NETPATH, API::Error::ERROR_BAD_NET_NAME, API::Error::ERROR_BAD_NET_RESP, API::Error::ERROR_BAD_PATHNAME, API::Error::ERROR_BAD_PIPE, API::Error::ERROR_BAD_PROFILE, API::Error::ERROR_BAD_PROVIDER, API::Error::ERROR_BAD_QUERY_SYNTAX, API::Error::ERROR_BAD_REM_ADAP, API::Error::ERROR_BAD_THREADID_ADDR, API::Error::ERROR_BAD_TOKEN_TYPE, API::Error::ERROR_BAD_UNIT, API::Error::ERROR_BAD_USERNAME, API::Error::ERROR_BAD_USER_PROFILE, API::Error::ERROR_BAD_VALIDATION_CLASS, API::Error::ERROR_BEGINNING_OF_MEDIA, API::Error::ERROR_BOOT_ALREADY_ACCEPTED, API::Error::ERROR_BROKEN_PIPE, API::Error::ERROR_BUSY, API::Error::ERROR_BUSY_DRIVE, API::Error::ERROR_BUS_RESET, API::Error::ERROR_CALLBACK_SUPPLIED_INVALID_DATA, API::Error::ERROR_CALL_NOT_IMPLEMENTED, API::Error::ERROR_CANCELLED, API::Error::ERROR_CANCEL_VIOLATION, API::Error::ERROR_CANNOT_COPY, API::Error::ERROR_CANNOT_DETECT_DRIVER_FAILURE, API::Error::ERROR_CANNOT_DETECT_PROCESS_ABORT, API::Error::ERROR_CANNOT_FIND_WND_CLASS, API::Error::ERROR_CANNOT_IMPERSONATE, API::Error::ERROR_CANNOT_MAKE, API::Error::ERROR_CANNOT_OPEN_PROFILE, API::Error::ERROR_CANTOPEN, API::Error::ERROR_CANTREAD, API::Error::ERROR_CANTWRITE, API::Error::ERROR_CANT_ACCESS_DOMAIN_INFO, API::Error::ERROR_CANT_ACCESS_FILE, API::Error::ERROR_CANT_DISABLE_MANDATORY, API::Error::ERROR_CANT_OPEN_ANONYMOUS, API::Error::ERROR_CANT_RESOLVE_FILENAME, API::Error::ERROR_CAN_NOT_COMPLETE, API::Error::ERROR_CHILD_ALIVE_NOWAIT, API::Error::ERROR_CHILD_MUST_BE_VOLATILE, API::Error::ERROR_CHILD_NOT_COMPLETE, API::Error::ERROR_CHILD_WINDOW_MENU, API::Error::ERROR_CIRCULAR_DEPENDENCY, API::Error::ERROR_CLASS_ALREADY_EXISTS, API::Error::ERROR_CLASS_DOES_NOT_EXIST, API::Error::ERROR_CLASS_HAS_WINDOWS, API::Error::ERROR_CLIPBOARD_NOT_OPEN, API::Error::ERROR_CLIPPING_NOT_SUPPORTED, API::Error::ERROR_COLORSPACE_MISMATCH, API::Error::ERROR_COMMITMENT_LIMIT, API::Error::ERROR_CONNECTED_OTHER_PASSWORD, API::Error::ERROR_CONNECTION_ABORTED, API::Error::ERROR_CONNECTION_ACTIVE, API::Error::ERROR_CONNECTION_COUNT_LIMIT, API::Error::ERROR_CONNECTION_INVALID, API::Error::ERROR_CONNECTION_REFUSED, API::Error::ERROR_CONNECTION_UNAVAIL, API::Error::ERROR_CONTEXT_EXPIRED, API::Error::ERROR_CONTINUE, API::Error::ERROR_CONTROL_ID_NOT_FOUND, API::Error::ERROR_COUNTER_TIMEOUT, API::Error::ERROR_CRC, API::Error::ERROR_CREATE_FAILED, API::Error::ERROR_CURRENT_DIRECTORY, API::Error::ERROR_CURRENT_DOMAIN_NOT_ALLOWED, API::Error::ERROR_DATABASE_DOES_NOT_EXIST, API::Error::ERROR_DATATYPE_MISMATCH, API::Error::ERROR_DC_NOT_FOUND, API::Error::ERROR_DDE_FAIL, API::Error::ERROR_DELETING_ICM_XFORM, API::Error::ERROR_DEPENDENT_SERVICES_RUNNING, API::Error::ERROR_DESTINATION_ELEMENT_FULL, API::Error::ERROR_DESTROY_OBJECT_OF_OTHER_THREAD, API::Error::ERROR_DEVICE_ALREADY_REMEMBERED, API::Error::ERROR_DEVICE_DOOR_OPEN, API::Error::ERROR_DEVICE_IN_USE, API::Error::ERROR_DEVICE_NOT_CONNECTED, API::Error::ERROR_DEVICE_NOT_PARTITIONED, API::Error::ERROR_DEVICE_REINITIALIZATION_NEEDED, API::Error::ERROR_DEVICE_REMOVED, API::Error::ERROR_DEVICE_REQUIRES_CLEANING, API::Error::ERROR_DEV_NOT_EXIST, API::Error::ERROR_DIFFERENT_SERVICE_ACCOUNT, API::Error::ERROR_DIRECTORY, API::Error::ERROR_DIRECT_ACCESS_HANDLE, API::Error::ERROR_DIR_NOT_EMPTY, API::Error::ERROR_DIR_NOT_ROOT, API::Error::ERROR_DISCARDED, API::Error::ERROR_DISK_CHANGE, API::Error::ERROR_DISK_CORRUPT, API::Error::ERROR_DISK_FULL, API::Error::ERROR_DISK_OPERATION_FAILED, API::Error::ERROR_DISK_RECALIBRATE_FAILED, API::Error::ERROR_DISK_RESET_FAILED, API::Error::ERROR_DISK_TOO_FRAGMENTED, API::Error::ERROR_DLL_INIT_FAILED, API::Error::ERROR_DLL_NOT_FOUND, API::Error::ERROR_DOMAIN_CONTROLLER_EXISTS, API::Error::ERROR_DOMAIN_CONTROLLER_NOT_FOUND, API::Error::ERROR_DOMAIN_EXISTS, API::Error::ERROR_DOMAIN_LIMIT_EXCEEDED, API::Error::ERROR_DOMAIN_TRUST_INCONSISTENT, API::Error::ERROR_DOWNGRADE_DETECTED, API::Error::ERROR_DRIVER_BLOCKED, API::Error::ERROR_DRIVE_LOCKED, API::Error::ERROR_DUPLICATE_SERVICE_NAME, API::Error::ERROR_DUPLICATE_TAG, API::Error::ERROR_DUP_DOMAINNAME, API::Error::ERROR_DUP_FCB, API::Error::ERROR_DUP_NAME, API::Error::ERROR_DYNLINK_FROM_INVALID_RING, API::Error::ERROR_EAS_DIDNT_FIT, API::Error::ERROR_EAS_NOT_SUPPORTED, API::Error::ERROR_EA_ACCESS_DENIED, API::Error::ERROR_EA_FILE_CORRUPT, API::Error::ERROR_EA_LIST_INCONSISTENT, API::Error::ERROR_EA_TABLE_FULL, API::Error::ERROR_END_OF_MEDIA, API::Error::ERROR_ENVVAR_NOT_FOUND, API::Error::ERROR_EOM_OVERFLOW, API::Error::ERROR_EVENTLOG_CANT_START, API::Error::ERROR_EVENTLOG_FILE_CHANGED, API::Error::ERROR_EVENTLOG_FILE_CORRUPT, API::Error::ERROR_EXCEPTION_IN_SERVICE, API::Error::ERROR_EXCL_SEM_ALREADY_OWNED, API::Error::ERROR_EXE_MACHINE_TYPE_MISMATCH, API::Error::ERROR_EXE_MARKED_INVALID, API::Error::ERROR_EXTENDED_ERROR, API::Error::ERROR_FAILED_SERVICE_CONTROLLER_CONNECT, API::Error::ERROR_FAIL_I24, API::Error::ERROR_FAIL_NOACTION_REBOOT, API::Error::ERROR_FAIL_RESTART, API::Error::ERROR_FAIL_SHUTDOWN, API::Error::ERROR_FCB_UNAVAILABLE, API::Error::ERROR_FILEMARK_DETECTED, API::Error::ERROR_FILENAME_EXCED_RANGE, API::Error::ERROR_FILE_CORRUPT, API::Error::ERROR_FILE_EXISTS, API::Error::ERROR_FILE_INVALID, API::Error::ERROR_FILE_NOT_FOUND, API::Error::ERROR_FLOPPY_BAD_REGISTERS, API::Error::ERROR_FLOPPY_ID_MARK_NOT_FOUND, API::Error::ERROR_FLOPPY_UNKNOWN_ERROR, API::Error::ERROR_FLOPPY_WRONG_CYLINDER, API::Error::ERROR_FULLSCREEN_MODE, API::Error::ERROR_FUNCTION_FAILED, API::Error::ERROR_FUNCTION_NOT_CALLED, API::Error::ERROR_GENERIC_NOT_MAPPED, API::Error::ERROR_GEN_FAILURE, API::Error::ERROR_GLOBAL_ONLY_HOOK, API::Error::ERROR_GRACEFUL_DISCONNECT, API::Error::ERROR_GROUP_EXISTS, API::Error::ERROR_HANDLE_DISK_FULL, API::Error::ERROR_HANDLE_EOF, API::Error::ERROR_HOOK_NEEDS_HMOD, API::Error::ERROR_HOOK_NOT_INSTALLED, API::Error::ERROR_HOOK_TYPE_NOT_ALLOWED, API::Error::ERROR_HOST_DOWN, API::Error::ERROR_HOST_UNREACHABLE, API::Error::ERROR_HOTKEY_ALREADY_REGISTERED, API::Error::ERROR_HOTKEY_NOT_REGISTERED, API::Error::ERROR_HWNDS_HAVE_DIFF_PARENT, API::Error::ERROR_ICM_NOT_ENABLED, API::Error::ERROR_ILLEGAL_ELEMENT_ADDRESS, API::Error::ERROR_ILL_FORMED_PASSWORD, API::Error::ERROR_INCORRECT_ADDRESS, API::Error::ERROR_INDEX_ABSENT, API::Error::ERROR_INFLOOP_IN_RELOC_CHAIN, API::Error::ERROR_INFO_NOT_AVAIL, API::Error::ERROR_INSTALL_ALREADY_RUNNING, API::Error::ERROR_INSTALL_FAILURE, API::Error::ERROR_INSTALL_LANGUAGE_UNSUPPORTED, API::Error::ERROR_INSTALL_LOG_FAILURE, API::Error::ERROR_INSTALL_NOTUSED, API::Error::ERROR_INSTALL_PACKAGE_INVALID, API::Error::ERROR_INSTALL_PACKAGE_OPEN_FAILED, API::Error::ERROR_INSTALL_PACKAGE_REJECTED, API::Error::ERROR_INSTALL_PACKAGE_VERSION, API::Error::ERROR_INSTALL_PLATFORM_UNSUPPORTED, API::Error::ERROR_INSTALL_REMOTE_DISALLOWED, API::Error::ERROR_INSTALL_SERVICE_FAILURE, API::Error::ERROR_INSTALL_SOURCE_ABSENT, API::Error::ERROR_INSTALL_SUSPEND, API::Error::ERROR_INSTALL_TEMP_UNWRITABLE, API::Error::ERROR_INSTALL_TRANSFORM_FAILURE, API::Error::ERROR_INSTALL_UI_FAILURE, API::Error::ERROR_INSTALL_USEREXIT, API::Error::ERROR_INSUFFICIENT_BUFFER, API::Error::ERROR_INTERNAL_DB_CORRUPTION, API::Error::ERROR_INTERNAL_DB_ERROR, API::Error::ERROR_INTERNAL_ERROR, API::Error::ERROR_INTERRUPT, API::Error::ERROR_INVALID_ACCEL_HANDLE, API::Error::ERROR_INVALID_ACCESS, API::Error::ERROR_INVALID_ACCOUNT_NAME, API::Error::ERROR_INVALID_ACL, API::Error::ERROR_INVALID_ADDRESS, API::Error::ERROR_INVALID_AT_INTERRUPT_TIME, API::Error::ERROR_INVALID_BLOCK, API::Error::ERROR_INVALID_BLOCK_LENGTH, API::Error::ERROR_INVALID_CALLGATE, API::Error::ERROR_INVALID_CATEGORY, API::Error::ERROR_INVALID_CMM, API::Error::ERROR_INVALID_COLORINDEX, API::Error::ERROR_INVALID_COLORSPACE, API::Error::ERROR_INVALID_COMBOBOX_MESSAGE, API::Error::ERROR_INVALID_COMMAND_LINE, API::Error::ERROR_INVALID_COMPUTERNAME, API::Error::ERROR_INVALID_CURSOR_HANDLE, API::Error::ERROR_INVALID_DATA, API::Error::ERROR_INVALID_DATATYPE, API::Error::ERROR_INVALID_DLL, API::Error::ERROR_INVALID_DOMAINNAME, API::Error::ERROR_INVALID_DOMAIN_ROLE, API::Error::ERROR_INVALID_DOMAIN_STATE, API::Error::ERROR_INVALID_DRIVE, API::Error::ERROR_INVALID_DWP_HANDLE, API::Error::ERROR_INVALID_EA_HANDLE, API::Error::ERROR_INVALID_EA_NAME, API::Error::ERROR_INVALID_EDIT_HEIGHT, API::Error::ERROR_INVALID_ENVIRONMENT, API::Error::ERROR_INVALID_EVENTNAME, API::Error::ERROR_INVALID_EVENT_COUNT, API::Error::ERROR_INVALID_EXE_SIGNATURE, API::Error::ERROR_INVALID_FIELD, API::Error::ERROR_INVALID_FILTER_PROC, API::Error::ERROR_INVALID_FLAGS, API::Error::ERROR_INVALID_FLAG_NUMBER, API::Error::ERROR_INVALID_FORM_NAME, API::Error::ERROR_INVALID_FORM_SIZE, API::Error::ERROR_INVALID_FUNCTION, API::Error::ERROR_INVALID_GROUPNAME, API::Error::ERROR_INVALID_GROUP_ATTRIBUTES, API::Error::ERROR_INVALID_GW_COMMAND, API::Error::ERROR_INVALID_HANDLE, API::Error::ERROR_INVALID_HANDLE_STATE, API::Error::ERROR_INVALID_HOOK_FILTER, API::Error::ERROR_INVALID_HOOK_HANDLE, API::Error::ERROR_INVALID_ICON_HANDLE, API::Error::ERROR_INVALID_ID_AUTHORITY, API::Error::ERROR_INVALID_IMPORT_OF_NON_DLL, API::Error::ERROR_INVALID_INDEX, API::Error::ERROR_INVALID_KEYBOARD_HANDLE, API::Error::ERROR_INVALID_LB_MESSAGE, API::Error::ERROR_INVALID_LEVEL, API::Error::ERROR_INVALID_LIST_FORMAT, API::Error::ERROR_INVALID_LOGON_HOURS, API::Error::ERROR_INVALID_LOGON_TYPE, API::Error::ERROR_INVALID_MEMBER, API::Error::ERROR_INVALID_MENU_HANDLE, API::Error::ERROR_INVALID_MESSAGE, API::Error::ERROR_INVALID_MESSAGEDEST, API::Error::ERROR_INVALID_MESSAGENAME, API::Error::ERROR_INVALID_MINALLOCSIZE, API::Error::ERROR_INVALID_MODULETYPE, API::Error::ERROR_INVALID_MONITOR_HANDLE, API::Error::ERROR_INVALID_MSGBOX_STYLE, API::Error::ERROR_INVALID_NAME, API::Error::ERROR_INVALID_NETNAME, API::Error::ERROR_INVALID_OPLOCK_PROTOCOL, API::Error::ERROR_INVALID_ORDINAL, API::Error::ERROR_INVALID_OWNER, API::Error::ERROR_INVALID_PARAMETER, API::Error::ERROR_INVALID_PASSWORD, API::Error::ERROR_INVALID_PASSWORDNAME, API::Error::ERROR_INVALID_PIXEL_FORMAT, API::Error::ERROR_INVALID_PRIMARY_GROUP, API::Error::ERROR_INVALID_PRINTER_COMMAND, API::Error::ERROR_INVALID_PRINTER_NAME, API::Error::ERROR_INVALID_PRINTER_STATE, API::Error::ERROR_INVALID_PRIORITY, API::Error::ERROR_INVALID_PROFILE, API::Error::ERROR_INVALID_SCROLLBAR_RANGE, API::Error::ERROR_INVALID_SECURITY_DESCR, API::Error::ERROR_INVALID_SEGDPL, API::Error::ERROR_INVALID_SEGMENT_NUMBER, API::Error::ERROR_INVALID_SEPARATOR_FILE, API::Error::ERROR_INVALID_SERVER_STATE, API::Error::ERROR_INVALID_SERVICENAME, API::Error::ERROR_INVALID_SERVICE_ACCOUNT, API::Error::ERROR_INVALID_SERVICE_CONTROL, API::Error::ERROR_INVALID_SERVICE_LOCK, API::Error::ERROR_INVALID_SHARENAME, API::Error::ERROR_INVALID_SHOWWIN_COMMAND, API::Error::ERROR_INVALID_SID, API::Error::ERROR_INVALID_SIGNAL_NUMBER, API::Error::ERROR_INVALID_SPI_VALUE, API::Error::ERROR_INVALID_STACKSEG, API::Error::ERROR_INVALID_STARTING_CODESEG, API::Error::ERROR_INVALID_SUB_AUTHORITY, API::Error::ERROR_INVALID_TABLE, API::Error::ERROR_INVALID_TARGET_HANDLE, API::Error::ERROR_INVALID_TASK_INDEX, API::Error::ERROR_INVALID_TASK_NAME, API::Error::ERROR_INVALID_THREAD_ID, API::Error::ERROR_INVALID_TIME, API::Error::ERROR_INVALID_TRANSFORM, API::Error::ERROR_INVALID_USER_BUFFER, API::Error::ERROR_INVALID_VERIFY_SWITCH, API::Error::ERROR_INVALID_WINDOW_HANDLE, API::Error::ERROR_INVALID_WINDOW_STYLE, API::Error::ERROR_INVALID_WORKSTATION, API::Error::ERROR_IOPL_NOT_ENABLED, API::Error::ERROR_IO_DEVICE, API::Error::ERROR_IO_INCOMPLETE, API::Error::ERROR_IO_PENDING, API::Error::ERROR_IRQ_BUSY, API::Error::ERROR_IS_JOINED, API::Error::ERROR_IS_JOIN_PATH, API::Error::ERROR_IS_JOIN_TARGET, API::Error::ERROR_IS_SUBSTED, API::Error::ERROR_IS_SUBST_PATH, API::Error::ERROR_IS_SUBST_TARGET, API::Error::ERROR_ITERATED_DATA_EXCEEDS_64k, API::Error::ERROR_JOIN_TO_JOIN, API::Error::ERROR_JOIN_TO_SUBST, API::Error::ERROR_JOURNAL_DELETE_IN_PROGRESS, API::Error::ERROR_JOURNAL_ENTRY_DELETED, API::Error::ERROR_JOURNAL_HOOK_SET, API::Error::ERROR_JOURNAL_NOT_ACTIVE, API::Error::ERROR_KEY_DELETED, API::Error::ERROR_KEY_HAS_CHILDREN, API::Error::ERROR_KM_DRIVER_BLOCKED, API::Error::ERROR_LABEL_TOO_LONG, API::Error::ERROR_LAST_ADMIN, API::Error::ERROR_LB_WITHOUT_TABSTOPS, API::Error::ERROR_LICENSE_QUOTA_EXCEEDED, API::Error::ERROR_LISTBOX_ID_NOT_FOUND, API::Error::ERROR_LM_CROSS_ENCRYPTION_REQUIRED, API::Error::ERROR_LOCAL_USER_SESSION_KEY, API::Error::ERROR_LOCKED, API::Error::ERROR_LOCK_FAILED, API::Error::ERROR_LOCK_VIOLATION, API::Error::ERROR_LOGIN_TIME_RESTRICTION, API::Error::ERROR_LOGIN_WKSTA_RESTRICTION, API::Error::ERROR_LOGON_FAILURE, API::Error::ERROR_LOGON_NOT_GRANTED, API::Error::ERROR_LOGON_SESSION_COLLISION, API::Error::ERROR_LOGON_SESSION_EXISTS, API::Error::ERROR_LOGON_TYPE_NOT_GRANTED, API::Error::ERROR_LOG_FILE_FULL, API::Error::ERROR_LUIDS_EXHAUSTED, API::Error::ERROR_MACHINE_LOCKED, API::Error::ERROR_MAGAZINE_NOT_PRESENT, API::Error::ERROR_MAPPED_ALIGNMENT, API::Error::ERROR_MAX_SESSIONS_REACHED, API::Error::ERROR_MAX_THRDS_REACHED, API::Error::ERROR_MEDIA_CHANGED, API::Error::ERROR_MEMBERS_PRIMARY_GROUP, API::Error::ERROR_MEMBER_IN_ALIAS, API::Error::ERROR_MEMBER_IN_GROUP, API::Error::ERROR_MEMBER_NOT_IN_ALIAS, API::Error::ERROR_MEMBER_NOT_IN_GROUP, API::Error::ERROR_MENU_ITEM_NOT_FOUND, API::Error::ERROR_MESSAGE_SYNC_ONLY, API::Error::ERROR_METAFILE_NOT_SUPPORTED, API::Error::ERROR_META_EXPANSION_TOO_LONG, API::Error::ERROR_MOD_NOT_FOUND, API::Error::ERROR_MONITORS_NOT_SUPPORTED, API::Error::ERROR_MORE_DATA, API::Error::ERROR_MORE_WRITES, API::Error::ERROR_MR_MID_NOT_FOUND, API::Error::ERROR_MUTUAL_AUTH_FAILED, API::Error::ERROR_NEGATIVE_SEEK, API::Error::ERROR_NESTING_NOT_ALLOWED, API::Error::ERROR_NETLOGON_NOT_STARTED, API::Error::ERROR_NETNAME_DELETED, API::Error::ERROR_NETWORK_ACCESS_DENIED, API::Error::ERROR_NETWORK_BUSY, API::Error::ERROR_NETWORK_UNREACHABLE, API::Error::ERROR_NET_WRITE_FAULT, API::Error::ERROR_NOACCESS, API::Error::ERROR_NOLOGON_INTERDOMAIN_TRUST_ACCOUNT, API::Error::ERROR_NOLOGON_SERVER_TRUST_ACCOUNT, API::Error::ERROR_NOLOGON_WORKSTATION_TRUST_ACCOUNT, API::Error::ERROR_NONE_MAPPED, API::Error::ERROR_NONPAGED_SYSTEM_RESOURCES, API::Error::ERROR_NON_MDICHILD_WINDOW, API::Error::ERROR_NOTIFY_ENUM_DIR, API::Error::ERROR_NOT_ALL_ASSIGNED, API::Error::ERROR_NOT_AUTHENTICATED, API::Error::ERROR_NOT_CHILD_WINDOW, API::Error::ERROR_NOT_CONNECTED, API::Error::ERROR_NOT_CONTAINER, API::Error::ERROR_NOT_CURRENT_CTRY, API::Error::ERROR_NOT_DOS_DISK, API::Error::ERROR_NOT_ENOUGH_MEMORY, API::Error::ERROR_NOT_ENOUGH_QUOTA, API::Error::ERROR_NOT_ENOUGH_SERVER_MEMORY, API::Error::ERROR_NOT_FOUND, API::Error::ERROR_NOT_FROZEN, API::Error::ERROR_NOT_JOINED, API::Error::ERROR_NOT_LOCKED, API::Error::ERROR_NOT_LOGGED_ON, API::Error::ERROR_NOT_LOGON_PROCESS, API::Error::ERROR_NOT_OWNER, API::Error::ERROR_NOT_READY, API::Error::ERROR_NOT_REGISTRY_FILE, API::Error::ERROR_NOT_SAME_DEVICE, API::Error::ERROR_NOT_SUBSTED, API::Error::ERROR_NOT_SUPPORTED, API::Error::ERROR_NOT_SUPPORTED_ON_SBS, API::Error::ERROR_NO_ASSOCIATION, API::Error::ERROR_NO_CHILD_PROCESS, API::Error::ERROR_NO_DATA, API::Error::ERROR_NO_DATA_DETECTED, API::Error::ERROR_NO_IMPERSONATION_TOKEN, API::Error::ERROR_NO_INHERITANCE, API::Error::ERROR_NO_ITEMS, API::Error::ERROR_NO_LOGON_SERVERS, API::Error::ERROR_NO_LOG_SPACE, API::Error::ERROR_NO_MATCH, API::Error::ERROR_NO_MEDIA_IN_DRIVE, API::Error::ERROR_NO_MORE_DEVICES, API::Error::ERROR_NO_MORE_FILES, API::Error::ERROR_NO_MORE_ITEMS, API::Error::ERROR_NO_MORE_SEARCH_HANDLES, API::Error::ERROR_NO_MORE_USER_HANDLES, API::Error::ERROR_NO_NETWORK, API::Error::ERROR_NO_NET_OR_BAD_PATH, API::Error::ERROR_NO_PROC_SLOTS, API::Error::ERROR_NO_QUOTAS_FOR_ACCOUNT, API::Error::ERROR_NO_RECOVERY_PROGRAM, API::Error::ERROR_NO_SCROLLBARS, API::Error::ERROR_NO_SECURITY_ON_OBJECT, API::Error::ERROR_NO_SHUTDOWN_IN_PROGRESS, API::Error::ERROR_NO_SIGNAL_SENT, API::Error::ERROR_NO_SITENAME, API::Error::ERROR_NO_SPOOL_SPACE, API::Error::ERROR_NO_SUCH_ALIAS, API::Error::ERROR_NO_SUCH_DOMAIN, API::Error::ERROR_NO_SUCH_GROUP, API::Error::ERROR_NO_SUCH_LOGON_SESSION, API::Error::ERROR_NO_SUCH_MEMBER, API::Error::ERROR_NO_SUCH_PACKAGE, API::Error::ERROR_NO_SUCH_PRIVILEGE, API::Error::ERROR_NO_SUCH_SITE, API::Error::ERROR_NO_SUCH_USER, API::Error::ERROR_NO_SYSTEM_MENU, API::Error::ERROR_NO_SYSTEM_RESOURCES, API::Error::ERROR_NO_TOKEN, API::Error::ERROR_NO_TRACKING_SERVICE, API::Error::ERROR_NO_TRUST_LSA_SECRET, API::Error::ERROR_NO_TRUST_SAM_ACCOUNT, API::Error::ERROR_NO_UNICODE_TRANSLATION, API::Error::ERROR_NO_USER_SESSION_KEY, API::Error::ERROR_NO_VOLUME_ID, API::Error::ERROR_NO_VOLUME_LABEL, API::Error::ERROR_NO_WILDCARD_CHARACTERS, API::Error::ERROR_NT_CROSS_ENCRYPTION_REQUIRED, API::Error::ERROR_NULL_LM_PASSWORD, API::Error::ERROR_OLD_WIN_VERSION, API::Error::ERROR_ONLY_IF_CONNECTED, API::Error::ERROR_OPEN_FAILED, API::Error::ERROR_OPEN_FILES, API::Error::ERROR_OPERATION_ABORTED, API::Error::ERROR_OPLOCK_NOT_GRANTED, API::Error::ERROR_OUT_OF_PAPER, API::Error::ERROR_OUT_OF_STRUCTURES, API::Error::ERROR_OVERRIDE_NOCHANGES, API::Error::ERROR_PAGED_SYSTEM_RESOURCES, API::Error::ERROR_PAGEFILE_QUOTA, API::Error::ERROR_PARTIAL_COPY, API::Error::ERROR_PARTITION_FAILURE, API::Error::ERROR_PASSWORD_EXPIRED, API::Error::ERROR_PASSWORD_MUST_CHANGE, API::Error::ERROR_PASSWORD_RESTRICTION, API::Error::ERROR_PATCH_PACKAGE_INVALID, API::Error::ERROR_PATCH_PACKAGE_OPEN_FAILED, API::Error::ERROR_PATCH_PACKAGE_UNSUPPORTED, API::Error::ERROR_PATH_BUSY, API::Error::ERROR_PATH_NOT_FOUND, API::Error::ERROR_PER_USER_TRUST_QUOTA_EXCEEDED, API::Error::ERROR_PIPE_BUSY, API::Error::ERROR_PIPE_CONNECTED, API::Error::ERROR_PIPE_LISTENING, API::Error::ERROR_PIPE_NOT_CONNECTED, API::Error::ERROR_PKINIT_FAILURE, API::Error::ERROR_POINT_NOT_FOUND, API::Error::ERROR_POPUP_ALREADY_ACTIVE, API::Error::ERROR_PORT_UNREACHABLE, API::Error::ERROR_POSSIBLE_DEADLOCK, API::Error::ERROR_POTENTIAL_FILE_FOUND, API::Error::ERROR_PRINTER_ALREADY_EXISTS, API::Error::ERROR_PRINTER_DELETED, API::Error::ERROR_PRINTER_DRIVER_ALREADY_INSTALLED, API::Error::ERROR_PRINTQ_FULL, API::Error::ERROR_PRINT_CANCELLED, API::Error::ERROR_PRIVATE_DIALOG_INDEX, API::Error::ERROR_PRIVILEGE_NOT_HELD, API::Error::ERROR_PROCESS_ABORTED, API::Error::ERROR_PROC_NOT_FOUND, API::Error::ERROR_PRODUCT_UNINSTALLED, API::Error::ERROR_PRODUCT_VERSION, API::Error::ERROR_PROFILE_NOT_ASSOCIATED_WITH_DEVICE, API::Error::ERROR_PROFILE_NOT_FOUND, API::Error::ERROR_PROTECTION_VIOLATION, API::Error::ERROR_PROTOCOL_UNREACHABLE, API::Error::ERROR_READ_FAULT, API::Error::ERROR_REDIRECTOR_HAS_OPEN_HANDLES, API::Error::ERROR_REDIR_PAUSED, API::Error::ERROR_REGISTRY_CORRUPT, API::Error::ERROR_REGISTRY_IO_FAILED, API::Error::ERROR_REGISTRY_RECOVERED, API::Error::ERROR_REG_NAT_CONSUMPTION, API::Error::ERROR_RELOC_CHAIN_XEEDS_SEGLIM, API::Error::ERROR_REMOTE_PRINT_CONNECTIONS_BLOCKED, API::Error::ERROR_REMOTE_SESSION_LIMIT_EXCEEDED, API::Error::ERROR_REM_NOT_LIST, API::Error::ERROR_REQUEST_ABORTED, API::Error::ERROR_REQUIRES_INTERACTIVE_WINDOWSTATION, API::Error::ERROR_REQ_NOT_ACCEP, API::Error::ERROR_RESOURCE_DATA_NOT_FOUND, API::Error::ERROR_RESOURCE_LANG_NOT_FOUND, API::Error::ERROR_RESOURCE_NAME_NOT_FOUND, API::Error::ERROR_RESOURCE_TYPE_NOT_FOUND, API::Error::ERROR_RETRY, API::Error::ERROR_REVISION_MISMATCH, API::Error::ERROR_RING2SEG_MUST_BE_MOVABLE, API::Error::ERROR_RING2_STACK_IN_USE, API::Error::ERROR_RMODE_APP, API::Error::ERROR_RXACT_COMMIT_FAILURE, API::Error::ERROR_RXACT_INVALID_STATE, API::Error::ERROR_SAME_DRIVE, API::Error::ERROR_SCOPE_NOT_FOUND, API::Error::ERROR_SCREEN_ALREADY_LOCKED, API::Error::ERROR_SECRET_TOO_LONG, API::Error::ERROR_SECTOR_NOT_FOUND, API::Error::ERROR_SEEK, API::Error::ERROR_SEEK_ON_DEVICE, API::Error::ERROR_SEM_IS_SET, API::Error::ERROR_SEM_NOT_FOUND, API::Error::ERROR_SEM_OWNER_DIED, API::Error::ERROR_SEM_TIMEOUT, API::Error::ERROR_SEM_USER_LIMIT, API::Error::ERROR_SERIAL_NO_DEVICE, API::Error::ERROR_SERVER_DISABLED, API::Error::ERROR_SERVER_HAS_OPEN_HANDLES, API::Error::ERROR_SERVER_NOT_DISABLED, API::Error::ERROR_SERVER_SHUTDOWN_IN_PROGRESS, API::Error::ERROR_SERVICE_ALREADY_RUNNING, API::Error::ERROR_SERVICE_CANNOT_ACCEPT_CTRL, API::Error::ERROR_SERVICE_DATABASE_LOCKED, API::Error::ERROR_SERVICE_DEPENDENCY_DELETED, API::Error::ERROR_SERVICE_DEPENDENCY_FAIL, API::Error::ERROR_SERVICE_DISABLED, API::Error::ERROR_SERVICE_DOES_NOT_EXIST, API::Error::ERROR_SERVICE_EXISTS, API::Error::ERROR_SERVICE_LOGON_FAILED, API::Error::ERROR_SERVICE_MARKED_FOR_DELETE, API::Error::ERROR_SERVICE_NEVER_STARTED, API::Error::ERROR_SERVICE_NOT_ACTIVE, API::Error::ERROR_SERVICE_NOT_FOUND, API::Error::ERROR_SERVICE_NOT_IN_EXE, API::Error::ERROR_SERVICE_NO_THREAD, API::Error::ERROR_SERVICE_REQUEST_TIMEOUT, API::Error::ERROR_SERVICE_SPECIFIC_ERROR, API::Error::ERROR_SERVICE_START_HANG, API::Error::ERROR_SESSION_CREDENTIAL_CONFLICT, API::Error::ERROR_SETCOUNT_ON_BAD_LB, API::Error::ERROR_SETMARK_DETECTED, API::Error::ERROR_SET_NOT_FOUND, API::Error::ERROR_SET_POWER_STATE_FAILED, API::Error::ERROR_SET_POWER_STATE_VETOED, API::Error::ERROR_SHARING_BUFFER_EXCEEDED, API::Error::ERROR_SHARING_PAUSED, API::Error::ERROR_SHARING_VIOLATION, API::Error::ERROR_SHUTDOWN_IN_PROGRESS, API::Error::ERROR_SIGNAL_PENDING, API::Error::ERROR_SIGNAL_REFUSED, API::Error::ERROR_SINGLE_INSTANCE_APP, API::Error::ERROR_SMARTCARD_SUBSYSTEM_FAILURE, API::Error::ERROR_SOME_NOT_MAPPED, API::Error::ERROR_SOURCE_ELEMENT_EMPTY, API::Error::ERROR_SPECIAL_ACCOUNT, API::Error::ERROR_SPECIAL_GROUP, API::Error::ERROR_SPECIAL_USER, API::Error::ERROR_STACK_OVERFLOW, API::Error::ERROR_SUBST_TO_JOIN, API::Error::ERROR_SUBST_TO_SUBST, API::Error::ERROR_SUCCESS, API::Error::ERROR_SUCCESS_REBOOT_INITIATED, API::Error::ERROR_SWAPERROR, API::Error::ERROR_SYNC_FOREGROUND_REFRESH_REQUIRED, API::Error::ERROR_SYSTEM_TRACE, API::Error::ERROR_TAG_NOT_FOUND, API::Error::ERROR_TAG_NOT_PRESENT, API::Error::ERROR_THREAD_1_INACTIVE, API::Error::ERROR_THREAD_ALREADY_IN_TASK, API::Error::ERROR_TIMEOUT, API::Error::ERROR_TIME_SKEW, API::Error::ERROR_TLW_WITH_WSCHILD, API::Error::ERROR_TOKEN_ALREADY_IN_USE, API::Error::ERROR_TOO_MANY_CMDS, API::Error::ERROR_TOO_MANY_CONTEXT_IDS, API::Error::ERROR_TOO_MANY_LINKS, API::Error::ERROR_TOO_MANY_LUIDS_REQUESTED, API::Error::ERROR_TOO_MANY_MODULES, API::Error::ERROR_TOO_MANY_MUXWAITERS, API::Error::ERROR_TOO_MANY_NAMES, API::Error::ERROR_TOO_MANY_OPEN_FILES, API::Error::ERROR_TOO_MANY_POSTS, API::Error::ERROR_TOO_MANY_SECRETS, API::Error::ERROR_TOO_MANY_SEMAPHORES, API::Error::ERROR_TOO_MANY_SEM_REQUESTS, API::Error::ERROR_TOO_MANY_SESS, API::Error::ERROR_TOO_MANY_SIDS, API::Error::ERROR_TOO_MANY_TCBS, API::Error::ERROR_TRANSFORM_NOT_SUPPORTED, API::Error::ERROR_TRUSTED_DOMAIN_FAILURE, API::Error::ERROR_TRUSTED_RELATIONSHIP_FAILURE, API::Error::ERROR_TRUST_FAILURE, API::Error::ERROR_UNABLE_TO_LOCK_MEDIA, API::Error::ERROR_UNABLE_TO_MOVE_REPLACEMENT, API::Error::ERROR_UNABLE_TO_MOVE_REPLACEMENT_2, API::Error::ERROR_UNABLE_TO_REMOVE_REPLACED, API::Error::ERROR_UNABLE_TO_UNLOAD_MEDIA, API::Error::ERROR_UNCERTAIN_MEDIA, API::Error::ERROR_UNEXP_NET_ERR, API::Error::ERROR_UNKNOWN_COMPONENT, API::Error::ERROR_UNKNOWN_FEATURE, API::Error::ERROR_UNKNOWN_PATCH, API::Error::ERROR_UNKNOWN_PORT, API::Error::ERROR_UNKNOWN_PRINTER_DRIVER, API::Error::ERROR_UNKNOWN_PRINTPROCESSOR, API::Error::ERROR_UNKNOWN_PRINT_MONITOR, API::Error::ERROR_UNKNOWN_PRODUCT, API::Error::ERROR_UNKNOWN_PROPERTY, API::Error::ERROR_UNKNOWN_REVISION, API::Error::ERROR_UNRECOGNIZED_MEDIA, API::Error::ERROR_UNRECOGNIZED_VOLUME, API::Error::ERROR_UNSUPPORTED_TYPE, API::Error::ERROR_USER_DEFINED_BASE, API::Error::ERROR_USER_DELETE_TRUST_QUOTA_EXCEEDED, API::Error::ERROR_USER_EXISTS, API::Error::ERROR_USER_MAPPED_FILE, API::Error::ERROR_USER_NOT_FOUND, API::Error::ERROR_USER_PROFILE_LOAD, API::Error::ERROR_VC_DISCONNECTED, API::Error::ERROR_VIOKBD_REQUEST, API::Error::ERROR_WAIT_NO_CHILDREN, API::Error::ERROR_WINDOW_NOT_COMBOBOX, API::Error::ERROR_WINDOW_NOT_DIALOG, API::Error::ERROR_WINDOW_OF_OTHER_THREAD, API::Error::ERROR_WORKING_SET_QUOTA, API::Error::ERROR_WRITE_FAULT, API::Error::ERROR_WRITE_PROTECT, API::Error::ERROR_WRONG_DISK, API::Error::ERROR_WRONG_PASSWORD, API::Error::ERROR_WRONG_TARGET_NAME, API::Error::ERR_TSTDUP, API::Error::ERR_TSTOVFL, API::Error::FORMAT_MESSAGE_ALLOCATE_BUFFER, API::Error::FORMAT_MESSAGE_ARGUMENT_ARRAY, API::Error::FORMAT_MESSAGE_FROM_HMODULE, API::Error::FORMAT_MESSAGE_FROM_STRING, API::Error::FORMAT_MESSAGE_FROM_SYSTEM, API::Error::FORMAT_MESSAGE_IGNORE_INSERTS, API::Error::FORMAT_MESSAGE_MAX_WIDTH_MASK, API::Error::LOAD_IGNORE_CODE_AUTHZ_LEVEL, API::Error::LOAD_LIBRARY_AS_DATAFILE, API::Error::LOAD_LIBRARY_AS_DATAFILE_EXCLUSIVE, API::Error::LOAD_LIBRARY_AS_IMAGE_RESOURCE, API::Error::LOAD_LIBRARY_SEARCH_APPLICATION_DIR, API::Error::LOAD_LIBRARY_SEARCH_DEFAULT_DIRS, API::Error::LOAD_LIBRARY_SEARCH_DLL_LOAD_DIR, API::Error::LOAD_LIBRARY_SEARCH_SYSTEM32, API::Error::LOAD_LIBRARY_SEARCH_USER_DIRS, API::Error::LOAD_WITH_ALTERED_SEARCH_PATH, API::Error::NO_ERROR, API::Error::OR_INVALID_OID, API::Error::OR_INVALID_OXID, API::Error::OR_INVALID_SET, API::Error::RPC_S_ADDRESS_ERROR, API::Error::RPC_S_ALREADY_LISTENING, API::Error::RPC_S_ALREADY_REGISTERED, API::Error::RPC_S_BINDING_HAS_NO_AUTH, API::Error::RPC_S_BINDING_INCOMPLETE, API::Error::RPC_S_CALL_CANCELLED, API::Error::RPC_S_CALL_FAILED, API::Error::RPC_S_CALL_FAILED_DNE, API::Error::RPC_S_CALL_IN_PROGRESS, API::Error::RPC_S_CANNOT_SUPPORT, API::Error::RPC_S_CANT_CREATE_ENDPOINT, API::Error::RPC_S_COMM_FAILURE, API::Error::RPC_S_DUPLICATE_ENDPOINT, API::Error::RPC_S_ENTRY_ALREADY_EXISTS, API::Error::RPC_S_ENTRY_NOT_FOUND, API::Error::RPC_S_ENTRY_TYPE_MISMATCH, API::Error::RPC_S_FP_DIV_ZERO, API::Error::RPC_S_FP_OVERFLOW, API::Error::RPC_S_FP_UNDERFLOW, API::Error::RPC_S_GROUP_MEMBER_NOT_FOUND, API::Error::RPC_S_GRP_ELT_NOT_ADDED, API::Error::RPC_S_GRP_ELT_NOT_REMOVED, API::Error::RPC_S_INCOMPLETE_NAME, API::Error::RPC_S_INTERFACE_NOT_EXPORTED, API::Error::RPC_S_INTERFACE_NOT_FOUND, API::Error::RPC_S_INTERNAL_ERROR, API::Error::RPC_S_INVALID_ASYNC_CALL, API::Error::RPC_S_INVALID_ASYNC_HANDLE, API::Error::RPC_S_INVALID_AUTH_IDENTITY, API::Error::RPC_S_INVALID_BINDING, API::Error::RPC_S_INVALID_BOUND, API::Error::RPC_S_INVALID_ENDPOINT_FORMAT, API::Error::RPC_S_INVALID_NAF_ID, API::Error::RPC_S_INVALID_NAME_SYNTAX, API::Error::RPC_S_INVALID_NETWORK_OPTIONS, API::Error::RPC_S_INVALID_NET_ADDR, API::Error::RPC_S_INVALID_OBJECT, API::Error::RPC_S_INVALID_RPC_PROTSEQ, API::Error::RPC_S_INVALID_STRING_BINDING, API::Error::RPC_S_INVALID_STRING_UUID, API::Error::RPC_S_INVALID_TAG, API::Error::RPC_S_INVALID_TIMEOUT, API::Error::RPC_S_INVALID_VERS_OPTION, API::Error::RPC_S_MAX_CALLS_TOO_SMALL, API::Error::RPC_S_NAME_SERVICE_UNAVAILABLE, API::Error::RPC_S_NOTHING_TO_EXPORT, API::Error::RPC_S_NOT_ALL_OBJS_EXPORTED, API::Error::RPC_S_NOT_ALL_OBJS_UNEXPORTED, API::Error::RPC_S_NOT_CANCELLED, API::Error::RPC_S_NOT_LISTENING, API::Error::RPC_S_NOT_RPC_ERROR, API::Error::RPC_S_NO_BINDINGS, API::Error::RPC_S_NO_CALL_ACTIVE, API::Error::RPC_S_NO_CONTEXT_AVAILABLE, API::Error::RPC_S_NO_ENDPOINT_FOUND, API::Error::RPC_S_NO_ENTRY_NAME, API::Error::RPC_S_NO_INTERFACES, API::Error::RPC_S_NO_MORE_BINDINGS, API::Error::RPC_S_NO_MORE_MEMBERS, API::Error::RPC_S_NO_PRINC_NAME, API::Error::RPC_S_NO_PROTSEQS, API::Error::RPC_S_NO_PROTSEQS_REGISTERED, API::Error::RPC_S_OBJECT_NOT_FOUND, API::Error::RPC_S_OUT_OF_RESOURCES, API::Error::RPC_S_PRF_ELT_NOT_ADDED, API::Error::RPC_S_PRF_ELT_NOT_REMOVED, API::Error::RPC_S_PROCNUM_OUT_OF_RANGE, API::Error::RPC_S_PROFILE_NOT_ADDED, API::Error::RPC_S_PROTOCOL_ERROR, API::Error::RPC_S_PROTSEQ_NOT_FOUND, API::Error::RPC_S_PROTSEQ_NOT_SUPPORTED, API::Error::RPC_S_SEC_PKG_ERROR, API::Error::RPC_S_SEND_INCOMPLETE, API::Error::RPC_S_SERVER_TOO_BUSY, API::Error::RPC_S_SERVER_UNAVAILABLE, API::Error::RPC_S_STRING_TOO_LONG, API::Error::RPC_S_TYPE_ALREADY_REGISTERED, API::Error::RPC_S_UNKNOWN_AUTHN_LEVEL, API::Error::RPC_S_UNKNOWN_AUTHN_SERVICE, API::Error::RPC_S_UNKNOWN_AUTHN_TYPE, API::Error::RPC_S_UNKNOWN_AUTHZ_SERVICE, API::Error::RPC_S_UNKNOWN_IF, API::Error::RPC_S_UNKNOWN_MGR_TYPE, API::Error::RPC_S_UNSUPPORTED_AUTHN_LEVEL, API::Error::RPC_S_UNSUPPORTED_NAME_SYNTAX, API::Error::RPC_S_UNSUPPORTED_TRANS_SYN, API::Error::RPC_S_UNSUPPORTED_TYPE, API::Error::RPC_S_UUID_LOCAL_ONLY, API::Error::RPC_S_UUID_NO_ADDRESS, API::Error::RPC_S_WRONG_KIND_OF_BINDING, API::Error::RPC_S_ZERO_DIVIDE, API::Error::RPC_X_BAD_STUB_DATA, API::Error::RPC_X_BYTE_COUNT_TOO_SMALL, API::Error::RPC_X_ENUM_VALUE_OUT_OF_RANGE, API::Error::RPC_X_INVALID_ES_ACTION, API::Error::RPC_X_INVALID_PIPE_OBJECT, API::Error::RPC_X_NO_MORE_ENTRIES, API::Error::RPC_X_NULL_REF_POINTER, API::Error::RPC_X_PIPE_CLOSED, API::Error::RPC_X_PIPE_DISCIPLINE_ERROR, API::Error::RPC_X_PIPE_EMPTY, API::Error::RPC_X_SS_CANNOT_GET_CALL_HANDLE, API::Error::RPC_X_SS_CHAR_TRANS_OPEN_FAIL, API::Error::RPC_X_SS_CHAR_TRANS_SHORT_FILE, API::Error::RPC_X_SS_CONTEXT_DAMAGED, API::Error::RPC_X_SS_HANDLES_MISMATCH, API::Error::RPC_X_SS_IN_NULL_CONTEXT, API::Error::RPC_X_WRONG_ES_VERSION, API::Error::RPC_X_WRONG_PIPE_ORDER, API::Error::RPC_X_WRONG_PIPE_VERSION, API::Error::RPC_X_WRONG_STUB_VERSION, API::Error::SEM_FAILCRITICALERRORS, API::Error::SEM_NOALIGNMENTFAULTEXCEPT, API::Error::SEM_NOGPFAULTERRORBOX, API::Error::SEM_NOOPENFILEERRORBOX, API::Error::S_OK

Instance Attribute Summary collapse

Class Method Summary collapse

Instance Method Summary collapse

Constructor Details

#initialize(pointer, owner = nil) ⇒ SID

Returns a new instance of SID.



37
38
39
40
41
# File 'lib/chef/win32/security/sid.rb', line 37

def initialize(pointer, owner = nil)
  @pointer = pointer
  # Keep a reference to the actual owner of this memory so we don't get freed
  @owner = owner
end

Instance Attribute Details

#pointer ⇒ Object (readonly)

Returns the value of attribute pointer.



56
57
58
# File 'lib/chef/win32/security/sid.rb', line 56

def pointer
  @pointer
end

Class Method Details

.AccountOperators ⇒ Object



204
205
206
# File 'lib/chef/win32/security/sid.rb', line 204

def self.AccountOperators
  SID.from_string_sid("S-1-5-32-548")
end

.admin_account_name ⇒ Object



261
262
263
264
265
266
267
268
269
270
271
272
273
274
275
276
277
278
279
280
281
282
283
284
285
286
287
288
289
290
291
292
293
294
295
296
297
298
299
300
# File 'lib/chef/win32/security/sid.rb', line 261

def self.
  @admin_account_name ||= begin
     = nil

    # Call NetUserEnum to enumerate the users without hitting network
    # http://msdn.microsoft.com/en-us/library/windows/desktop/aa370652(v=vs.85).aspx
    servername = nil # We are querying the local server
    level = 3 # We want USER_INFO_3 structure which contains the SID
    filter = FILTER_NORMAL_ACCOUNT # Only query the user accounts
    bufptr = FFI::MemoryPointer.new(:pointer) # Buffer which will receive the data
    prefmaxlen = MAX_PREFERRED_LENGTH # Let the system allocate the needed amount of memory
    entriesread = FFI::Buffer.new(:long).write_long(0)
    totalentries = FFI::Buffer.new(:long).write_long(0)
    resume_handle = FFI::Buffer.new(:long).write_long(0)

    status = ERROR_MORE_DATA

    while status == ERROR_MORE_DATA
      status = NetUserEnum(servername, level, filter, bufptr, prefmaxlen, entriesread, totalentries, resume_handle)

      if status == NERR_Success || status == ERROR_MORE_DATA
        entriesread.read_long.times.collect do |i|
           = USER_INFO_3.new(bufptr.read_pointer + i * USER_INFO_3.size)
          # Check if the account is the Administrator account
          # RID for the Administrator account is always 500 and it's privilage is set to USER_PRIV_ADMIN
          if [:usri3_user_id] == 500 && [:usri3_priv] == 2 # USER_PRIV_ADMIN (2) - Administrator
             = [:usri3_name].read_wstring
            break
          end
        end

        # Free the memory allocated by the system
        NetApiBufferFree(bufptr.read_pointer)
      end
    end

    raise "Can not determine the administrator account name." if .nil?
    
  end
end

.Administrator ⇒ Object



232
233
234
# File 'lib/chef/win32/security/sid.rb', line 232

def self.Administrator
  SID.("#{::ENV['COMPUTERNAME']}\\#{SID.}")
end

.Administrators ⇒ Object



224
225
226
# File 'lib/chef/win32/security/sid.rb', line 224

def self.Administrators
  SID.from_string_sid("S-1-5-32-544")
end

.Anonymous ⇒ Object



148
149
150
# File 'lib/chef/win32/security/sid.rb', line 148

def self.Anonymous
  SID.from_string_sid("S-1-5-7")
end

.AuthenticatedUsers ⇒ Object



164
165
166
# File 'lib/chef/win32/security/sid.rb', line 164

def self.AuthenticatedUsers
  SID.from_string_sid("S-1-5-11")
end

.BackupOperators ⇒ Object



216
217
218
# File 'lib/chef/win32/security/sid.rb', line 216

def self.BackupOperators
  SID.from_string_sid("S-1-5-32-551")
end

.Batch ⇒ Object



136
137
138
# File 'lib/chef/win32/security/sid.rb', line 136

def self.Batch
  SID.from_string_sid("S-1-5-3")
end

.BuiltinAdministrators ⇒ Object



188
189
190
# File 'lib/chef/win32/security/sid.rb', line 188

def self.BuiltinAdministrators
  SID.from_string_sid("S-1-5-32-544")
end

.BuiltinUsers ⇒ Object



192
193
194
# File 'lib/chef/win32/security/sid.rb', line 192

def self.BuiltinUsers
  SID.from_string_sid("S-1-5-32-545")
end

.Creator ⇒ Object



100
101
102
# File 'lib/chef/win32/security/sid.rb', line 100

def self.Creator
  SID.from_string_sid("S-1-3")
end

.CreatorGroup ⇒ Object



108
109
110
# File 'lib/chef/win32/security/sid.rb', line 108

def self.CreatorGroup
  SID.from_string_sid("S-1-3-1")
end

.CreatorGroupServer ⇒ Object



116
117
118
# File 'lib/chef/win32/security/sid.rb', line 116

def self.CreatorGroupServer
  SID.from_string_sid("S-1-3-3")
end

.CreatorOwner ⇒ Object



104
105
106
# File 'lib/chef/win32/security/sid.rb', line 104

def self.CreatorOwner
  SID.from_string_sid("S-1-3-0")
end

.CreatorOwnerServer ⇒ Object



112
113
114
# File 'lib/chef/win32/security/sid.rb', line 112

def self.CreatorOwnerServer
  SID.from_string_sid("S-1-3-2")
end

.current_user ⇒ Object



240
241
242
# File 'lib/chef/win32/security/sid.rb', line 240

def self.current_user
  SID.("#{::ENV['USERDOMAIN']}\\#{::ENV['USERNAME']}")
end

.default_security_object_group ⇒ Object

See https://technet.microsoft.com/en-us/library/cc961996.aspx In practice, this seems to be SID.current_user for Microsoft Accounts, the current user's Domain Users group for domain accounts, and SID.None otherwise.



256
257
258
259
# File 'lib/chef/win32/security/sid.rb', line 256

def self.default_security_object_group
  token = Chef::ReservedNames::Win32::Security.open_current_process_token
  Chef::ReservedNames::Win32::Security.get_token_information_primary_group(token)
end

.default_security_object_owner ⇒ Object

See https://technet.microsoft.com/en-us/library/cc961992.aspx In practice, this is SID.Administrators if the current_user is an admin (even if not running elevated), and is current_user otherwise. On win2k3, it technically can be current_user in all cases if a certain group policy is set.



248
249
250
251
# File 'lib/chef/win32/security/sid.rb', line 248

def self.default_security_object_owner
  token = Chef::ReservedNames::Win32::Security.open_current_process_token
  Chef::ReservedNames::Win32::Security.get_token_information_owner(token)
end

.Dialup ⇒ Object



128
129
130
# File 'lib/chef/win32/security/sid.rb', line 128

def self.Dialup
  SID.from_string_sid("S-1-5-1")
end

.EnterpriseDomainControllers ⇒ Object



156
157
158
# File 'lib/chef/win32/security/sid.rb', line 156

def self.EnterpriseDomainControllers
  SID.from_string_sid("S-1-5-9")
end

.Everyone ⇒ Object



92
93
94
# File 'lib/chef/win32/security/sid.rb', line 92

def self.Everyone
  SID.from_string_sid("S-1-1-0")
end

.from_account(name) ⇒ Object



43
44
45
46
# File 'lib/chef/win32/security/sid.rb', line 43

def self.(name)
  domain, sid, use = Chef::ReservedNames::Win32::Security.(name)
  sid
end

.from_string_sid(string_sid) ⇒ Object



48
49
50
# File 'lib/chef/win32/security/sid.rb', line 48

def self.from_string_sid(string_sid)
  Chef::ReservedNames::Win32::Security.convert_string_sid_to_sid(string_sid)
end

.Guest ⇒ Object



236
237
238
# File 'lib/chef/win32/security/sid.rb', line 236

def self.Guest
  SID.("#{::ENV['COMPUTERNAME']}\\Guest")
end

.Guests ⇒ Object



196
197
198
# File 'lib/chef/win32/security/sid.rb', line 196

def self.Guests
  SID.from_string_sid("S-1-5-32-546")
end

.Interactive ⇒ Object



140
141
142
# File 'lib/chef/win32/security/sid.rb', line 140

def self.Interactive
  SID.from_string_sid("S-1-5-4")
end

.Local ⇒ Object



96
97
98
# File 'lib/chef/win32/security/sid.rb', line 96

def self.Local
  SID.from_string_sid("S-1-2")
end

.LocalSystem ⇒ Object



176
177
178
# File 'lib/chef/win32/security/sid.rb', line 176

def self.LocalSystem
  SID.from_string_sid("S-1-5-18")
end

.Network ⇒ Object



132
133
134
# File 'lib/chef/win32/security/sid.rb', line 132

def self.Network
  SID.from_string_sid("S-1-5-2")
end

.Nobody ⇒ Object



84
85
86
# File 'lib/chef/win32/security/sid.rb', line 84

def self.Nobody
  SID.from_string_sid("S-1-0-0")
end

.None ⇒ Object



228
229
230
# File 'lib/chef/win32/security/sid.rb', line 228

def self.None
  SID.("#{::ENV['COMPUTERNAME']}\\None")
end

.NonUnique ⇒ Object



120
121
122
# File 'lib/chef/win32/security/sid.rb', line 120

def self.NonUnique
  SID.from_string_sid("S-1-4")
end

.Nt ⇒ Object



124
125
126
# File 'lib/chef/win32/security/sid.rb', line 124

def self.Nt
  SID.from_string_sid("S-1-5")
end

.NtLocal ⇒ Object



180
181
182
# File 'lib/chef/win32/security/sid.rb', line 180

def self.NtLocal
  SID.from_string_sid("S-1-5-19")
end

.NtNetwork ⇒ Object



184
185
186
# File 'lib/chef/win32/security/sid.rb', line 184

def self.NtNetwork
  SID.from_string_sid("S-1-5-20")
end

.Null ⇒ Object

Well-known SIDs



80
81
82
# File 'lib/chef/win32/security/sid.rb', line 80

def self.Null
  SID.from_string_sid("S-1-0")
end

.PowerUsers ⇒ Object



200
201
202
# File 'lib/chef/win32/security/sid.rb', line 200

def self.PowerUsers
  SID.from_string_sid("S-1-5-32-547")
end

.PrincipalSelf ⇒ Object



160
161
162
# File 'lib/chef/win32/security/sid.rb', line 160

def self.PrincipalSelf
  SID.from_string_sid("S-1-5-10")
end

.PrintOperators ⇒ Object



212
213
214
# File 'lib/chef/win32/security/sid.rb', line 212

def self.PrintOperators
  SID.from_string_sid("S-1-5-32-550")
end

.Proxy ⇒ Object



152
153
154
# File 'lib/chef/win32/security/sid.rb', line 152

def self.Proxy
  SID.from_string_sid("S-1-5-8")
end

.Replicators ⇒ Object



220
221
222
# File 'lib/chef/win32/security/sid.rb', line 220

def self.Replicators
  SID.from_string_sid("S-1-5-32-552")
end

.RestrictedCode ⇒ Object



168
169
170
# File 'lib/chef/win32/security/sid.rb', line 168

def self.RestrictedCode
  SID.from_string_sid("S-1-5-12")
end

.ServerOperators ⇒ Object



208
209
210
# File 'lib/chef/win32/security/sid.rb', line 208

def self.ServerOperators
  SID.from_string_sid("S-1-5-32-549")
end

.Service ⇒ Object



144
145
146
# File 'lib/chef/win32/security/sid.rb', line 144

def self.Service
  SID.from_string_sid("S-1-5-6")
end

.TerminalServerUsers ⇒ Object



172
173
174
# File 'lib/chef/win32/security/sid.rb', line 172

def self.TerminalServerUsers
  SID.from_string_sid("S-1-5-13")
end

.World ⇒ Object



88
89
90
# File 'lib/chef/win32/security/sid.rb', line 88

def self.World
  SID.from_string_sid("S-1-1")
end

Instance Method Details

#==(other) ⇒ Object



52
53
54
# File 'lib/chef/win32/security/sid.rb', line 52

def ==(other)
  other != nil && Chef::ReservedNames::Win32::Security.equal_sid(self, other)
end

#account ⇒ Object



58
59
60
# File 'lib/chef/win32/security/sid.rb', line 58

def 
  Chef::ReservedNames::Win32::Security.(self)
end

#account_name ⇒ Object



62
63
64
65
# File 'lib/chef/win32/security/sid.rb', line 62

def 
  domain, name, use = 
  (domain != nil && domain.length > 0) ? "#{domain}\\#{name}" : name
end

#size ⇒ Object



67
68
69
# File 'lib/chef/win32/security/sid.rb', line 67

def size
  Chef::ReservedNames::Win32::Security.get_length_sid(self)
end

#to_s ⇒ Object



71
72
73
# File 'lib/chef/win32/security/sid.rb', line 71

def to_s
  Chef::ReservedNames::Win32::Security.convert_sid_to_string_sid(self)
end

#valid? ⇒ Boolean

Returns:

  • (Boolean)


75
76
77
# File 'lib/chef/win32/security/sid.rb', line 75

def valid?
  Chef::ReservedNames::Win32::Security.is_valid_sid(self)
end