Class: Chef::Resource::Service

Inherits:
Chef::Resource show all
Defined in:
lib/chef/resource/service.rb

Constant Summary

Constants inherited from Chef::Resource

FORBIDDEN_IVARS, HIDDEN_IVARS

Instance Attribute Summary

Attributes inherited from Chef::Resource

#allowed_actions, #cookbook_name, #delayed_notifications, #enclosing_provider, #immediate_notifications, #not_if_args, #only_if_args, #params, #provider, #recipe_name, #resource_name, #retries, #retry_delay, #run_context, #source_line, #updated

Instance Method Summary collapse

Methods inherited from Chef::Resource

#action, #add_notification, #after_created, #as_json, attribute, build_from_file, #defined_at, #epic_fail, #ignore_failure, #inspect, #is, json_create, #load_prior_resource, #method_missing, #name, #node, #noop, #not_if, #notifies, #notifies_delayed, #notifies_immediately, #only_if, platform_map, provider_base, provides, #resolve_notification_references, resource_for_node, resource_for_platform, #resources, #run_action, #should_skip?, #subscribes, #to_hash, #to_json, #to_s, #to_text, #updated?, #updated_by_last_action, #updated_by_last_action?

Methods included from Mixin::ConvertToClassName

#convert_to_class_name, #convert_to_snake_case, #filename_to_qualified_string, #snake_case_basename

Methods included from Mixin::Deprecation

#deprecated_ivar

Methods included from Mixin::Language

#data_bag, #data_bag_item, #platform?, #platform_family?, #search, #value_for_platform, #value_for_platform_family

Methods included from Mixin::ParamsValidate

#set_or_return, #validate

Methods included from Mixin::CheckHelper

#set_if_args

Constructor Details

#initialize(name, run_context = nil) ⇒ Service



25
26
27
28
29
30
31
32
33
34
35
36
37
38
39
40
41
42
43
# File 'lib/chef/resource/service.rb', line 25

def initialize(name, run_context=nil)
  super
  @resource_name = :service
  @service_name = name
  @enabled = nil
  @running = nil
  @parameters = nil
  @pattern = service_name
  @start_command = nil
  @stop_command = nil
  @status_command = nil
  @restart_command = nil
  @reload_command = nil
  @priority = nil
  @action = "nothing"
  @startup_type = :automatic
  @supports = { :restart => false, :reload => false, :status => false }
  @allowed_actions.push(:enable, :disable, :start, :stop, :restart, :reload)
end

Dynamic Method Handling

This class handles dynamic methods through the method_missing method in the class Chef::Resource

Instance Method Details

#enabled(arg = nil) ⇒ Object

if the service is enabled or not



107
108
109
110
111
112
113
# File 'lib/chef/resource/service.rb', line 107

def enabled(arg=nil)
  set_or_return(
    :enabled,
    arg,
    :kind_of => [ TrueClass, FalseClass ]
  )
end

#parameters(arg = nil) ⇒ Object



140
141
142
143
144
145
# File 'lib/chef/resource/service.rb', line 140

def parameters(arg=nil)
  set_or_return(
    :parameters,
    arg,
    :kind_of => [ Hash ] )
end

#pattern(arg = nil) ⇒ Object

regex for match against ps -ef when !supports && status == nil



54
55
56
57
58
59
60
# File 'lib/chef/resource/service.rb', line 54

def pattern(arg=nil)
  set_or_return(
    :pattern,
    arg,
    :kind_of => [ String ]
  )
end

#priority(arg = nil) ⇒ Object

Priority arguments can have two forms:

  • a simple number, in which the default start runlevels get that as the start value and stop runlevels get 100 - value.

  • a hash like { 2 => [:start, 20], 3 => [:stop, 55] }, where the service will be marked as started with priority 20 in runlevel 2, stopped in 3 with priority 55 and no symlinks or similar for other runlevels



134
135
136
137
138
# File 'lib/chef/resource/service.rb', line 134

def priority(arg=nil)
  set_or_return(:priority,
                arg,
                :kind_of => [ Integer, String, Hash ])
end

#reload_command(arg = nil) ⇒ Object



98
99
100
101
102
103
104
# File 'lib/chef/resource/service.rb', line 98

def reload_command(arg=nil)
  set_or_return(
    :reload_command,
    arg,
    :kind_of => [ String ]
  )
end

#restart_command(arg = nil) ⇒ Object

command to call to restart service



90
91
92
93
94
95
96
# File 'lib/chef/resource/service.rb', line 90

def restart_command(arg=nil)
  set_or_return(
    :restart_command,
    arg,
    :kind_of => [ String ]
  )
end

#running(arg = nil) ⇒ Object

if the service is running or not



116
117
118
119
120
121
122
# File 'lib/chef/resource/service.rb', line 116

def running(arg=nil)
  set_or_return(
    :running,
    arg,
    :kind_of => [ TrueClass, FalseClass ]
  )
end

#service_name(arg = nil) ⇒ Object



45
46
47
48
49
50
51
# File 'lib/chef/resource/service.rb', line 45

def service_name(arg=nil)
  set_or_return(
    :service_name,
    arg,
    :kind_of => [ String ]
  )
end

#start_command(arg = nil) ⇒ Object

command to call to start service



63
64
65
66
67
68
69
# File 'lib/chef/resource/service.rb', line 63

def start_command(arg=nil)
  set_or_return(
    :start_command,
    arg,
    :kind_of => [ String ]
  )
end

#status_command(arg = nil) ⇒ Object

command to call to get status of service



81
82
83
84
85
86
87
# File 'lib/chef/resource/service.rb', line 81

def status_command(arg=nil)
  set_or_return(
    :status_command,
    arg,
    :kind_of => [ String ]
  )
end

#stop_command(arg = nil) ⇒ Object

command to call to stop service



72
73
74
75
76
77
78
# File 'lib/chef/resource/service.rb', line 72

def stop_command(arg=nil)
  set_or_return(
    :stop_command,
    arg,
    :kind_of => [ String ]
  )
end

#supports(args = {}) ⇒ Object



147
148
149
150
151
152
153
154
155
# File 'lib/chef/resource/service.rb', line 147

def supports(args={})
  if args.is_a? Array
    args.each { |arg| @supports[arg] = true }
  elsif args.any?
    @supports = args
  else
    @supports
  end
end