Class: Chef::Win32::API::Security::ACLStruct
- Defined in:
- lib/chef/win32/api/security.rb
Overview
ACL type is a header with some information, followed by an array of ACEs msdn.microsoft.com/en-us/library/windows/desktop/aa374931(v=VS.85).aspx
ACL type is a header with some information, followed by an array of ACEs msdn.microsoft.com/en-us/library/windows/desktop/aa374931(v=VS.85).aspx