Method: Chef::Cookbook::CookbookVersionLoader#load
- Defined in:
- lib/chef/cookbook/cookbook_version_loader.rb
#load ⇒ Object Also known as: load_cookbooks
84 85 86 87 88 89 90 91 92 93 94 95 96 97 98 99 100 |
# File 'lib/chef/cookbook/cookbook_version_loader.rb', line 84 def load Chef.deprecated(:internal_api, "Chef::Cookbook::CookbookVersionLoader's load method is deprecated. Please use load! instead.") # force lazy evaluation to occur # re-raise any exception that occurred when reading the metadata load_all_files remove_ignored_files if empty? Chef::Log.warn "Found a directory #{cookbook_name} in the cookbook path, but it contains no cookbook files. skipping." end cookbook_settings end |