Method: Chef::CookbookVersion#frozen_version?
- Defined in:
- lib/chef/cookbook_version.rb
#frozen_version? ⇒ Boolean
Indicates if this version is frozen or not. Freezing a cookbook version indicates that a new cookbook with the same name and version number should
114 115 116 |
# File 'lib/chef/cookbook_version.rb', line 114 def frozen_version? @frozen end |