Method: Chef::CookbookVersion#metadata
- Defined in:
- lib/chef/cookbook_version.rb
#metadata ⇒ Object
A Chef::Cookbook::Metadata object. It has a setter that fixes up the metadata to add descriptions of the recipes contained in this CookbookVersion.
55 56 57 |
# File 'lib/chef/cookbook_version.rb', line 55 def end |