Method: Chef::EventDispatch::Base#action_collection_registration
- Defined in:
- lib/chef/event_dispatch/base.rb
#action_collection_registration(action_collection) ⇒ Object
Callback hook for handlers to grab a reference to the action_collection (sent before compiling cookbooks, consumers can also find it off the run_context.action_collection)
268 |
# File 'lib/chef/event_dispatch/base.rb', line 268 def action_collection_registration(action_collection); end |