Method: Chef::EventDispatch::Base#resource_update_applied
- Defined in:
- lib/chef/event_dispatch/base.rb
#resource_update_applied(resource, action, update) ⇒ Object
Called when a change has been made to a resource. May be called multiple times per resource, e.g., a file may have its content updated, and then its permissions updated.
328 |
# File 'lib/chef/event_dispatch/base.rb', line 328 def resource_update_applied(resource, action, update); end |