Method: Chef::EventDispatch::Base#resource_update_progress
- Defined in:
- lib/chef/event_dispatch/base.rb
#resource_update_progress(resource, current, total, interval) ⇒ Object
Called when a progress notification should be sent to the user to indicate the overall progress of a long running operation, such as a large file download.
333 |
# File 'lib/chef/event_dispatch/base.rb', line 333 def resource_update_progress(resource, current, total, interval); end |