Method: Chef::Formatters::Base#indent_by
- Defined in:
- lib/chef/formatters/base.rb
#indent_by(amount) ⇒ Object
94 95 96 97 98 99 100 101 102 103 |
# File 'lib/chef/formatters/base.rb', line 94 def indent_by(amount) @output.indent += amount if @output.indent < 0 # This is left commented out for now. We need to uncomment it and fix at least one bug in # the formatter, and then leave this line uncommented in the future. # Chef::Log.warn "Internal Formatter Error -- Attempt to indent by negative number of spaces" @output.indent = 0 end @output.indent end |