Method: Chef::Node::Attribute#debug_value
- Defined in:
- lib/chef/node/attribute.rb
#debug_value(*args) ⇒ Object
Debug what’s going on with an attribute. args is a path spec to the attribute you’re interested in. For example, to debug where the value of node[:network][:default_interface] is coming from, use:
debug_value(:network, :default_interface).
The return value is an Array of Arrays. The Arrays are pairs of ‘[“precedence_level”, value]`, where precedence level is the component, such as role default, normal, etc. and value is the attribute value set at that precedence level. If there is no value at that precedence level, value will be the symbol :not_present.
225 226 227 228 229 230 231 232 233 234 235 |
# File 'lib/chef/node/attribute.rb', line 225 def debug_value(*args) COMPONENTS.map do |component| value = begin instance_variable_get(component).read!(*args) rescue :not_present end [component.to_s.sub(/^@/, ""), value] end end |