Method: Chef::Node#consume_ohai_data
- Defined in:
- lib/chef/node.rb
#consume_ohai_data(ohai_data) ⇒ Object
This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.
This is for ohai plugins to consume ohai data and have it merged, it should probably be renamed
369 370 371 372 |
# File 'lib/chef/node.rb', line 369 def consume_ohai_data(ohai_data) self.automatic_attrs = Chef::Mixin::DeepMerge.merge(automatic_attrs, ohai_data) fix_automatic_attributes end |