Method: Chef::Node#run_list
- Defined in:
- lib/chef/node.rb
#run_list(*args) ⇒ Object
Returns an Array of roles and recipes, in the order they will be applied. If you call it with arguments, they will become the new list of roles and recipes.
339 340 341 342 |
# File 'lib/chef/node.rb', line 339 def run_list(*args) rl = select_run_list args.length > 0 ? rl.reset!(args) : rl end |