Method: Chef::Provider::Execute#define_resource_requirements
- Defined in:
- lib/chef/provider/execute.rb
#define_resource_requirements ⇒ Object
37 38 39 40 41 42 |
# File 'lib/chef/provider/execute.rb', line 37 def define_resource_requirements if creates && creates_relative? && !cwd # FIXME? move this onto the resource? raise Chef::Exceptions::Execute, "Please either specify a full path for the creates property, or specify a cwd property to the #{new_resource} resource" end end |