Method: Chef::Provider#action
- Defined in:
- lib/chef/provider.rb
#action ⇒ Object
TODO:
this should be a reader, and the action should be passed in the
constructor; however, many/most subclasses override the constructor so changing the arity would be a breaking change. Change this at the next major release
127 128 129 |
# File 'lib/chef/provider.rb', line 127 def action @action end |