Method: Chef::Provider#whyrun_supported?
- Defined in:
- lib/chef/provider.rb
#whyrun_supported? ⇒ Boolean
as of Chef 13 we enable why-run by default and require resources to override this to set false. We’re keeping this method to prevent breaking the cookbook world for no real gain on our part.
159 160 161 |
# File 'lib/chef/provider.rb', line 159 def whyrun_supported? true end |