Method: Chef::Resource#elapsed_time
- Defined in:
- lib/chef/resource.rb
#elapsed_time ⇒ Integer (readonly)
The time it took (in seconds) to run the most recently-run action. Not cumulative across actions. This is set to 0 as soon as a new action starts running, and set to the elapsed time at the end of the action.
action. Not cumulative.
470 471 472 |
# File 'lib/chef/resource.rb', line 470 def elapsed_time @elapsed_time end |