Method: Chef::Resource#source_line
- Defined in:
- lib/chef/resource.rb
#source_line ⇒ String
Returns The source line where this resource was declared. Expected to come from caller() or a stack trace, it usually follows one of these formats:
/some/path/to/file.rb:80:in `wombat_tears'
C:/some/path/to/file.rb:80 in 1`wombat_tears'.
1265 1266 1267 |
# File 'lib/chef/resource.rb', line 1265 def source_line @source_line end |