Method: Chef::RunContext::CookbookCompiler#compile
- Defined in:
- lib/chef/run_context/cookbook_compiler.rb
#compile ⇒ Object
Run the compile phase of the chef run. Loads files in the following order:
-
Libraries
-
Ohai
-
Compliance Profiles/Waivers
-
Attributes
-
LWRPs
-
Resource Definitions
-
Recipes
Recipes are loaded in precisely the order specified by the expanded run_list.
Other files are loaded in an order derived from the expanded run_list and the dependencies declared by cookbooks’ metadata. See #cookbook_order for more information.
102 103 104 105 106 107 108 109 110 |
# File 'lib/chef/run_context/cookbook_compiler.rb', line 102 def compile compile_libraries compile_ohai_plugins compile_compliance compile_attributes compile_lwrps compile_resource_definitions compile_recipes end |