Method: Chef::RunContext#request_reboot
- Defined in:
- lib/chef/run_context.rb
#request_reboot(reboot_info) ⇒ Object
there are options for how to handle multiple calls to these functions:
-
first call always wins (never change reboot_info once set).
-
last call always wins (happily change reboot_info whenever).
-
raise an exception on the first conflict.
-
disable reboot after this run if anyone ever calls :cancel.
-
raise an exception on any second call.
-
?
640 641 642 643 |
# File 'lib/chef/run_context.rb', line 640 def request_reboot(reboot_info) logger.info "Changing reboot status from #{self.reboot_info.inspect} to #{reboot_info.inspect}" @reboot_info = reboot_info end |