Method: Chef::RunContext#updated_resources
- Defined in:
- lib/chef/run_context.rb
#updated_resources ⇒ Object (readonly)
A Set keyed by the string name, of all the resources that are updated. We do not track actions or individual resource objects, since this matches the behavior of the notification collections which are keyed by Strings.
201 202 203 |
# File 'lib/chef/run_context.rb', line 201 def updated_resources @updated_resources end |