Class: Chef::Provider::Service::Macosx
- Inherits:
-
Simple
- Object
- Chef::Provider
- Chef::Provider::Service
- Simple
- Chef::Provider::Service::Macosx
- Includes:
- Mixin::ShellOut
- Defined in:
- lib/chef/provider/service/macosx.rb
Constant Summary collapse
- PLIST_DIRS =
gather_plist_dirs
Constants included from Mixin::ShellOut
Mixin::ShellOut::DEPRECATED_OPTIONS
Instance Attribute Summary
Attributes inherited from Chef::Provider
#action, #cookbook_name, #current_resource, #new_resource, #recipe_name, #run_context
Class Method Summary collapse
Instance Method Summary collapse
- #define_resource_requirements ⇒ Object
- #disable_service ⇒ Object
-
#enable_service ⇒ Object
On OS/X, enabling a service has the side-effect of starting it, and disabling a service has the side-effect of stopping it.
- #load_current_resource ⇒ Object
- #restart_service ⇒ Object
- #set_service_status ⇒ Object
- #start_service ⇒ Object
- #stop_service ⇒ Object
Methods included from Mixin::ShellOut
#run_command_compatible_options, #shell_out, #shell_out!
Methods inherited from Simple
#reload_service, #shared_resource_requirements, #whyrun_supported?
Methods inherited from Chef::Provider::Service
#action_disable, #action_enable, #action_reload, #action_restart, #action_start, #action_stop, #initialize, #load_new_resource_state, #reload_service, #shared_resource_requirements, #whyrun_supported?
Methods included from Mixin::Command
#chdir_or_tmpdir, #handle_command_failures, #output_of_command, #run_command, #run_command_and_return_stdout_stderr, #run_command_with_systems_locale
Methods included from Mixin::Command::Windows
Methods included from Mixin::Command::Unix
Methods inherited from Chef::Provider
#action_nothing, #cleanup_after_converge, #converge_by, #events, #initialize, #node, #process_resource_requirements, #requirements, #resource_collection, #resource_updated?, #run_action, #set_updated_status, #whyrun_mode?, #whyrun_supported?
Methods included from DSL::Recipe
#build_resource, #declare_resource, #describe_self_for_error, #evaluate_resource_definition, #has_resource_definition?, #have_resource_class_for?, #method_missing, #resource_class_for
Methods included from Mixin::ConvertToClassName
#convert_to_class_name, #convert_to_snake_case, #filename_to_qualified_string, #snake_case_basename
Constructor Details
This class inherits a constructor from Chef::Provider::Service
Dynamic Method Handling
This class handles dynamic methods through the method_missing method in the class Chef::DSL::Recipe
Class Method Details
.gather_plist_dirs ⇒ Object
28 29 30 31 32 33 34 35 36 |
# File 'lib/chef/provider/service/macosx.rb', line 28 def self.gather_plist_dirs locations = %w{/Library/LaunchAgents /Library/LaunchDaemons /System/Library/LaunchAgents /System/Library/LaunchDaemons } locations << "#{ENV['HOME']}/Library/LaunchAgents" if ENV['HOME'] locations end |
Instance Method Details
#define_resource_requirements ⇒ Object
51 52 53 54 55 56 57 58 59 60 61 62 63 64 65 66 67 68 69 70 71 72 73 74 75 76 77 |
# File 'lib/chef/provider/service/macosx.rb', line 51 def define_resource_requirements #super requirements.assert(:reload) do |a| a. Chef::Exceptions::UnsupportedAction, "#{self.to_s} does not support :reload" end requirements.assert(:all_actions) do |a| a.assertion { @plist_size < 2 } a. Chef::Exceptions::Service, "Several plist files match service name. Please use full service name." end requirements.assert(:all_actions) do |a| a.assertion { !@service_label.to_s.empty? } a. Chef::Exceptions::Service, "Could not find service's label in plist file '#{@plist}'!" end requirements.assert(:all_actions) do |a| a.assertion { @plist_size > 0 } # No failrue here in original code - so we also will not # fail. Instead warn that the service is potentially missing a.whyrun "Assuming that the service would have been previously installed and is currently disabled." do @current_resource.enabled(false) @current_resource.running(false) end end end |
#disable_service ⇒ Object
129 130 131 132 133 134 135 136 137 138 |
# File 'lib/chef/provider/service/macosx.rb', line 129 def disable_service unless @current_resource.enabled Chef::Log.debug("#{@new_resource} not enabled, not disabling") else shell_out!( "launchctl unload -w '#{@plist}'", :user => @owner_uid, :group => @owner_gid ) end end |
#enable_service ⇒ Object
On OS/X, enabling a service has the side-effect of starting it, and disabling a service has the side-effect of stopping it.
This makes some sense on OS/X since launchctl is an “init”-style supervisor that will restart daemons that are crashing, etc.
118 119 120 121 122 123 124 125 126 127 |
# File 'lib/chef/provider/service/macosx.rb', line 118 def enable_service if @current_resource.enabled Chef::Log.debug("#{@new_resource} already enabled, not enabling") else shell_out!( "launchctl load -w '#{@plist}'", :user => @owner_uid, :group => @owner_gid ) end end |
#load_current_resource ⇒ Object
40 41 42 43 44 45 46 47 48 49 |
# File 'lib/chef/provider/service/macosx.rb', line 40 def load_current_resource @current_resource = Chef::Resource::Service.new(@new_resource.name) @current_resource.service_name(@new_resource.service_name) @plist_size = 0 @plist = find_service_plist @service_label = find_service_label set_service_status @current_resource end |
#restart_service ⇒ Object
103 104 105 106 107 108 109 110 111 |
# File 'lib/chef/provider/service/macosx.rb', line 103 def restart_service if @new_resource.restart_command super else stop_service sleep 1 start_service end end |
#set_service_status ⇒ Object
140 141 142 143 144 145 146 147 148 149 150 151 152 153 154 155 156 157 158 159 160 161 162 163 164 165 166 167 168 169 170 |
# File 'lib/chef/provider/service/macosx.rb', line 140 def set_service_status return if @plist == nil or @service_label.to_s.empty? cmd = shell_out( "launchctl list #{@service_label}", :user => @owner_uid, :group => @owner_gid ) if cmd.exitstatus == 0 @current_resource.enabled(true) else @current_resource.enabled(false) end if @current_resource.enabled @owner_uid = ::File.stat(@plist).uid @owner_gid = ::File.stat(@plist).gid shell_out!( "launchctl list", :user => @owner_uid, :group => @owner_gid ).stdout.each_line do |line| case line when /(\d+|-)\s+(?:\d+|-)\s+(.*\.?)#{@service_label}/ pid = $1 @current_resource.running(!pid.to_i.zero?) end end else @current_resource.running(false) end end |
#start_service ⇒ Object
79 80 81 82 83 84 85 86 87 88 89 |
# File 'lib/chef/provider/service/macosx.rb', line 79 def start_service if @current_resource.running Chef::Log.debug("#{@new_resource} already running, not starting") else if @new_resource.start_command super else shell_out!("launchctl load -w '#{@plist}'", :user => @owner_uid, :group => @owner_gid) end end end |
#stop_service ⇒ Object
91 92 93 94 95 96 97 98 99 100 101 |
# File 'lib/chef/provider/service/macosx.rb', line 91 def stop_service unless @current_resource.running Chef::Log.debug("#{@new_resource} not running, not stopping") else if @new_resource.stop_command super else shell_out!("launchctl unload '#{@plist}'", :user => @owner_uid, :group => @owner_gid) end end end |