Method: Chef::REST#get_rest
- Defined in:
- lib/chef/rest.rb
#get_rest(path, raw = false, headers = {}) ⇒ Object
Send an HTTP GET request to the path
Using this method to fetch a file is considered deprecated.
Parameters
- path
-
The path to GET
- raw
-
Whether you want the raw body returned, or JSON inflated. Defaults
to JSON inflated.
110 111 112 113 114 115 116 |
# File 'lib/chef/rest.rb', line 110 def get_rest(path, raw=false, headers={}) if raw streaming_request(create_url(path), headers) else api_request(:GET, create_url(path), headers) end end |